@@ -28,6 +28,8 @@
 #include "rb-debug.h"
 
 #define RB_TREE_DND_STRING "RbTreeDndString"
+/* must be the same value as in gtk_tree_view.c */
+#define SCROLL_EDGE_SIZE 15
 
 typedef struct
 {
@@ -53,8 +55,17 @@ typedef struct
   GdkDragAction source_actions;
   GdkModifierType start_button_mask;
 
+  /* Scroll timeout (e.g. during dnd) */
+  guint scroll_timeout;
+
 } RbTreeDndData;
 
+RbTreeDndData *init_rb_tree_dnd_data (GtkWidget *widget);
+GList * get_context_data (GdkDragContext *context);
+static gboolean filter_drop_position (GtkWidget *widget, GdkDragContext *context, GtkTreePath *path, GtkTreeViewDropPosition *pos);
+static gint scroll_row_timeout (gpointer data);
+static void remove_scroll_timeout (GtkTreeView *tree_view);
+
 GType
 rb_tree_drag_source_get_type (void)
 {
@@ -243,9 +254,6 @@ rb_tree_drag_dest_row_drop_position (RbTreeDragDest   *drag_dest,
   return (* iface->row_drop_position) (drag_dest, dest_path, targets, pos);
 }
 
-
-
-
 RbTreeDndData *
 init_rb_tree_dnd_data (GtkWidget *widget)
 {
@@ -259,13 +267,12 @@ init_rb_tree_dnd_data (GtkWidget *widget)
 		g_object_set_data (G_OBJECT (widget), RB_TREE_DND_STRING, priv_data);
 		priv_data->drag_leave_handler = 0;
 		priv_data->button_press_event_handler = 0;
+		priv_data->scroll_timeout = 0;
 	}
 
 	return priv_data;
 }
 
-
-
 static void
 stop_drag_check (GtkWidget *widget)
 {
@@ -324,7 +331,6 @@ path_list_free (GList *path_list)
 	g_list_free (path_list);
 }
 
-
 static void
 set_context_data (GdkDragContext *context,
 		  GList          *path_list)
@@ -337,46 +343,109 @@ set_context_data (GdkDragContext *context,
 	rb_debug ("Setting path_list: index=%i", gtk_tree_path_get_indices(path_list->data)[0]);
 }
 
-
 GList *
 get_context_data (GdkDragContext *context)
 {
 	return g_object_get_data (G_OBJECT (context), "rb-tree-view-multi-source-row");
 }
 
-
-void
+gboolean
 filter_drop_position (GtkWidget *widget, GdkDragContext *context, GtkTreePath *path, GtkTreeViewDropPosition *pos)
 {
-  GtkTreeView *tree_view = GTK_TREE_VIEW (widget);
-  GtkTreeModel *model = gtk_tree_view_get_model (tree_view);
-  RbTreeDndData *priv_data = g_object_get_data (G_OBJECT (widget), RB_TREE_DND_STRING);
+	GtkTreeView *tree_view = GTK_TREE_VIEW (widget);
+	GtkTreeModel *model = gtk_tree_view_get_model (tree_view);
+	RbTreeDndData *priv_data = g_object_get_data (G_OBJECT (widget), RB_TREE_DND_STRING);
+	gboolean ret;
 
-  if (!(priv_data->dest_flags &
-	(RB_TREE_DEST_CAN_DROP_INTO | RB_TREE_DEST_CAN_DROP_BETWEEN))) {
+	if (!(priv_data->dest_flags & RB_TREE_DEST_CAN_DROP_INTO)) {
+		if (*pos == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE)
+			*pos = GTK_TREE_VIEW_DROP_BEFORE;
+		else if (*pos == GTK_TREE_VIEW_DROP_INTO_OR_AFTER)
+			*pos = GTK_TREE_VIEW_DROP_AFTER;
+	} else if (!(priv_data->dest_flags & RB_TREE_DEST_CAN_DROP_BETWEEN)) {
+		if (*pos == GTK_TREE_VIEW_DROP_BEFORE)
+			*pos = GTK_TREE_VIEW_DROP_INTO_OR_BEFORE;
+		else if (*pos == GTK_TREE_VIEW_DROP_AFTER)
+			*pos = GTK_TREE_VIEW_DROP_INTO_OR_AFTER;
+	}
 
-    rb_tree_drag_dest_row_drop_position (RB_TREE_DRAG_DEST (model),
-					 path,
-					 context->targets,
-					 pos);
-
-  } else {
-    if (!(priv_data->dest_flags & RB_TREE_DEST_CAN_DROP_INTO)) {
-      if (*pos == GTK_TREE_VIEW_DROP_INTO_OR_BEFORE)
-	*pos = GTK_TREE_VIEW_DROP_BEFORE;
-      else if (*pos == GTK_TREE_VIEW_DROP_INTO_OR_AFTER)
-	*pos = GTK_TREE_VIEW_DROP_AFTER;
-
-    } else if (!(priv_data->dest_flags & RB_TREE_DEST_CAN_DROP_BETWEEN)) {
-      if (*pos == GTK_TREE_VIEW_DROP_BEFORE)
-	*pos = GTK_TREE_VIEW_DROP_INTO_OR_BEFORE;
-      else if (*pos == GTK_TREE_VIEW_DROP_AFTER)
-	*pos = GTK_TREE_VIEW_DROP_INTO_OR_AFTER;
-    }
-  }
+	ret = rb_tree_drag_dest_row_drop_position (RB_TREE_DRAG_DEST (model),
+						   path,
+						   context->targets,
+						   pos);
+  
+	rb_debug ("filtered drop position: %s", ret ? "TRUE" : "FALSE");	
+	return ret;
 }
 
 
+/* Scroll function taken/adapted from gtktreeview.c */
+static gint
+scroll_row_timeout (gpointer data)
+{
+	GtkTreeView *tree_view = data;
+	GdkRectangle visible_rect;
+	gint y, x;
+	gint offset;
+	gfloat value;
+	GtkAdjustment* vadj;
+	RbTreeDndData *priv_data;
+
+	GDK_THREADS_ENTER ();
+
+	priv_data = g_object_get_data (G_OBJECT (tree_view), RB_TREE_DND_STRING);
+	g_return_val_if_fail(priv_data != NULL, TRUE);
+
+	gdk_window_get_pointer (gtk_tree_view_get_bin_window (tree_view), &x, &y, NULL);
+	gtk_tree_view_widget_to_tree_coords (tree_view, x, y, &x, &y);
+	gtk_tree_view_get_visible_rect (tree_view, &visible_rect);
+
+	/* see if we are near the edge. */
+	if (x < visible_rect.x && x > visible_rect.x + visible_rect.width)
+	{
+		GDK_THREADS_LEAVE ();
+		priv_data->scroll_timeout = 0;
+		return FALSE;
+	}
+
+	offset = y - (visible_rect.y + 2 * SCROLL_EDGE_SIZE);
+	if (offset > 0)
+	{
+		offset = y - (visible_rect.y + visible_rect.height - 2 * SCROLL_EDGE_SIZE);
+		if (offset < 0) 
+		{
+			GDK_THREADS_LEAVE ();
+			priv_data->scroll_timeout = 0;
+			return FALSE;
+		}
+	}
+
+	vadj = gtk_tree_view_get_vadjustment (tree_view);
+	value = CLAMP (vadj->value + offset, vadj->lower, vadj->upper - vadj->page_size);
+	gtk_adjustment_set_value (vadj, value);
+
+	GDK_THREADS_LEAVE ();
+
+	return TRUE;
+}
+
+
+
+static void
+remove_scroll_timeout (GtkTreeView *tree_view)
+ {
+	 RbTreeDndData *priv_data;
+
+	 priv_data = g_object_get_data (G_OBJECT (tree_view), RB_TREE_DND_STRING);
+	 g_return_if_fail(priv_data != NULL);
+
+	 if (priv_data->scroll_timeout != 0)
+	 {
+		 g_source_remove (priv_data->scroll_timeout);
+		 priv_data->scroll_timeout = 0;
+	 }
+}
+
 
 static void
 rb_tree_dnd_drag_data_delete_cb (GtkWidget *widget,
@@ -513,7 +582,15 @@ rb_tree_dnd_drag_motion_cb (GtkWidget        *widget,
 	}
 	else
 	{
-	  filter_drop_position (widget, context, path, &pos);
+		if (!filter_drop_position (widget, context, path, &pos)) {
+			gdk_drag_status (context, 0, time);
+			return TRUE;
+		}
+
+	  if (priv_data->scroll_timeout == 0)
+	  {
+		  priv_data->scroll_timeout = g_timeout_add (150, scroll_row_timeout, tree_view);
+	  }
 	}
 
 	if (GTK_WIDGET (tree_view) == gtk_drag_get_source_widget (context) &&
@@ -548,11 +625,11 @@ rb_tree_dnd_drag_drop_cb (GtkWidget        *widget,
 
 	tree_view = GTK_TREE_VIEW (widget);
 	model = gtk_tree_view_get_model (tree_view);
-
 	priv_data = g_object_get_data (G_OBJECT (widget), RB_TREE_DND_STRING);
-
 	gtk_tree_view_get_dest_row_at_pos (tree_view, x, y, &path, &pos);
 
+	remove_scroll_timeout (tree_view);
+
 	/* Unset this thing */
 	gtk_tree_view_set_drag_dest_row (tree_view,
                				 NULL,
@@ -587,6 +664,7 @@ rb_tree_dnd_drag_data_received_cb (GtkWidget        *widget,
 	GtkTreeModel *model;
 	GtkTreePath *dest_row;
 	GtkTreeViewDropPosition pos;
+	gboolean filtered = TRUE;
 	gboolean accepted = FALSE;
 
 	tree_view = GTK_TREE_VIEW (widget);
@@ -595,9 +673,10 @@ rb_tree_dnd_drag_data_received_cb (GtkWidget        *widget,
 	gtk_tree_view_get_dest_row_at_pos (tree_view, x, y, &dest_row, &pos);
 
 	if (dest_row)
-		filter_drop_position (widget, context, dest_row, &pos);
+		if (!filter_drop_position (widget, context, dest_row, &pos))
+			filtered = FALSE;
 
-	if (selection_data->length >= 0)
+	if (filtered && selection_data->length >= 0)
 	{
 		if (rb_tree_drag_dest_drag_data_received (RB_TREE_DRAG_DEST (model),
                 					  dest_row,
