@@ -55,7 +55,7 @@ static void rhythmdb_tree_save (RhythmDB *rdb);
 static RhythmDBEntry * rhythmdb_tree_entry_new (RhythmDB *db, RhythmDBEntryType type,
 						const char *uri);
 static void rhythmdb_tree_entry_set (RhythmDB *db, RhythmDBEntry *entry,
-				guint propid, GValue *value);
+				     guint propid, const GValue *value);
 
 static void rhythmdb_tree_entry_get (RhythmDB *db, RhythmDBEntry *entry,
 				guint propid, GValue *value);
@@ -222,6 +222,30 @@ rhythmdb_tree_init (RhythmDBTree *db)
 
 }
 
+void
+rhythmdb_entry_unref (RhythmDB *adb, RhythmDBEntry *aentry)
+{
+	RhythmDBTree *db = (RhythmDBTree *) adb;
+	RhythmDBTreeEntry *entry = (RhythmDBTreeEntry *) aentry;
+
+	if (rb_atomic_dec (&entry->refcount) <= 1) {
+		rhythmdb_write_lock (adb);
+		rhythmdb_tree_entry_destroy (db, entry);
+		rhythmdb_write_unlock (adb);
+	}
+}
+
+void
+rhythmdb_entry_unref_unlocked (RhythmDB *adb, RhythmDBEntry *aentry)
+{
+	RhythmDBTree *db = (RhythmDBTree *) adb;
+	RhythmDBTreeEntry *entry = (RhythmDBTreeEntry *) aentry;
+
+	if (rb_atomic_dec (&entry->refcount) <= 1) {
+		rhythmdb_tree_entry_destroy (db, entry);
+	}
+}
+
 static inline char *
 get_entry_genre_name (RhythmDBTreeEntry *entry)
 {
@@ -279,14 +303,14 @@ get_entry_album_folded (RhythmDBTreeEntry *entry)
 static inline void
 sanity_check_entry_tree (RhythmDBTreeEntry *entry)
 {
-#ifdef RHYTHMDB_ENABLE_SANITY_CHECK
+#if defined(RHYTHMDB_ENABLE_SANITY_CHECK) && RHYTHMDB_ENABLE_SANITY_CHECK > 1
 	RHYTHMDB_TREE_ENTRY (entry); 
 	RHYTHMDB_TREE_PROPERTY (entry->album); 
 	RHYTHMDB_TREE_PROPERTY (entry->album->parent); 
 	RHYTHMDB_TREE_PROPERTY (entry->album->parent->parent); 
 #endif
 }
-#ifdef RHYTHMDB_ENABLE_SANITY_CHECK
+#if defined(RHYTHMDB_ENABLE_SANITY_CHECK) && RHYTHMDB_ENABLE_SANITY_CHECK > 1
 static void
 sanity_check_entry_tree_from_hash (gpointer unused, RhythmDBTreeEntry *entry)
 {
@@ -296,7 +320,7 @@ sanity_check_entry_tree_from_hash (gpointer unused, RhythmDBTreeEntry *entry)
 static void
 sanity_check_database (RhythmDBTree *db)
 {
-#ifdef RHYTHMDB_ENABLE_SANITY_CHECK
+#if defined(RHYTHMDB_ENABLE_SANITY_CHECK) && RHYTHMDB_ENABLE_SANITY_CHECK > 1
 	g_hash_table_foreach (db->priv->entries, (GHFunc) sanity_check_entry_tree_from_hash, NULL);
 #endif
 }
@@ -517,12 +541,16 @@ rhythmdb_tree_parser_end_element (struct RhythmDBTreeLoadContext *ctx, const cha
 			g_value_set_long (value, g_ascii_strtoull (ctx->buf->str, NULL, 10));
 			g_string_free (ctx->buf, TRUE);
 			break;
+		case G_TYPE_UINT64:
+			g_value_set_uint64 (value, g_ascii_strtoull (ctx->buf->str, NULL, 10));
+			g_string_free (ctx->buf, TRUE);
+			break;
 		case G_TYPE_FLOAT:
 			g_value_set_float (value, g_ascii_strtod (ctx->buf->str, NULL));
 			g_string_free (ctx->buf, TRUE);
 			break;
 		case G_TYPE_DOUBLE:
-			g_value_set_float (value, g_ascii_strtod (ctx->buf->str, NULL));
+			g_value_set_double (value, g_ascii_strtod (ctx->buf->str, NULL));
 			g_string_free (ctx->buf, TRUE);
 			break;
 		default:
@@ -666,6 +694,21 @@ g_free (encoded);							\
 			if (g_value_get_long (value) == 0)
 				continue;
 			break;
+		case G_TYPE_FLOAT:
+		{
+			float v = g_value_get_float (value);
+			/* We don't care about precision so much. */
+			if (v > -0.001 && v < 0.001)
+				continue;
+			break;
+		}
+		case G_TYPE_DOUBLE:
+		{
+			double v = g_value_get_double (value);
+			if (v > -0.001 && v < 0.001)
+				continue;
+			break;
+		}
 		case G_TYPE_BOOLEAN:
 			if (g_value_get_boolean (value) == FALSE)
 				continue;
@@ -708,6 +751,9 @@ g_free (encoded);							\
 		case G_TYPE_LONG:
 			RHYTHMDB_FWRITE_SMALLTYPE (ctx->handle, "%ld", long);
 			break;
+		case G_TYPE_UINT64:
+			RHYTHMDB_FWRITE_SMALLTYPE (ctx->handle, "%llu", uint64);
+			break;
 		case G_TYPE_FLOAT:
 			RHYTHMDB_FWRITE_SMALLTYPE (ctx->handle, "%f", float);
 			break;
@@ -1053,7 +1099,7 @@ remove_entry_from_album (RhythmDBTree *db, RhythmDBTreeEntry *entry)
 
 static void
 rhythmdb_tree_entry_set (RhythmDB *adb, RhythmDBEntry *aentry,
-			 guint propid, GValue *value)
+			 guint propid, const GValue *value)
 {
 	RhythmDBTree *db = RHYTHMDB_TREE (adb);
 	RhythmDBTreeEntry *entry = RHYTHMDB_TREE_ENTRY (aentry);
@@ -1220,6 +1266,18 @@ rhythmdb_tree_entry_get (RhythmDB *adb, RhythmDBEntry *aentry,
 	case RHYTHMDB_PROP_LAST_PLAYED_STR:
 		g_value_set_static_string (value, g_value_get_string (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_LAST_PLAYED_STR)));
 		break;
+	case RHYTHMDB_PROP_TRACK_GAIN:
+		g_value_set_double (value, g_value_get_double (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_TRACK_GAIN)));
+		break;
+	case RHYTHMDB_PROP_TRACK_PEAK:
+		g_value_set_double (value, g_value_get_double (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_TRACK_PEAK)));
+		break;
+	case RHYTHMDB_PROP_ALBUM_GAIN:
+		g_value_set_double (value, g_value_get_double (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_ALBUM_GAIN)));
+		break;
+	case RHYTHMDB_PROP_ALBUM_PEAK:
+		g_value_set_double (value, g_value_get_double (RHYTHMDB_TREE_ENTRY_VALUE (entry, RHYTHMDB_PROP_ALBUM_PEAK)));
+		break;
 	default:
 		g_value_copy (RHYTHMDB_TREE_ENTRY_VALUE (entry, propid), value);
 		break;
@@ -1249,8 +1307,10 @@ rhythmdb_tree_entry_delete (RhythmDB *adb, RhythmDBEntry *aentry)
 
 	sanity_check_database (db);
 
-	if (entry->deleted)
+	if (entry->deleted) {
+		rb_debug ("entry %x was already deleted", aentry);
 		return;
+	}
 
 	entry->deleted = TRUE;
 
