@@ -1,7 +1,7 @@
 /*
  *  arch-tag: Header for RhythmDB - Rhythmbox backend queryable database
  *
- *  Copyright (C) 2003 Colin Walters <walters@rhythmbox.org>
+ *  Copyright (C) 2003,2004 Colin Walters <walters@rhythmbox.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -59,8 +59,8 @@ typedef enum
 	RHYTHMDB_QUERY_PROP_LESS,
 } RhythmDBQueryType;
 
-#define RHYTHMDB_NUM_PROPERTIES 23
-#define RHYTHMDB_NUM_SAVED_PROPERTIES 14
+#define RHYTHMDB_NUM_PROPERTIES 28
+#define RHYTHMDB_NUM_SAVED_PROPERTIES 19
 
 typedef enum
 {
@@ -77,12 +77,17 @@ typedef enum
 	RHYTHMDB_PROP_RATING,
 	RHYTHMDB_PROP_PLAY_COUNT,
 	RHYTHMDB_PROP_LAST_PLAYED,
-	RHYTHMDB_PROP_QUALITY,
+	RHYTHMDB_PROP_BITRATE,
+	RHYTHMDB_PROP_TRACK_GAIN,
+	RHYTHMDB_PROP_TRACK_PEAK,
+	RHYTHMDB_PROP_ALBUM_GAIN,
+	RHYTHMDB_PROP_ALBUM_PEAK,
+	RHYTHMDB_PROP_MIMETYPE,
 } RhythmDBPropType;
 
 typedef enum
 {
-	RHYTHMDB_PROP_TITLE_SORT_KEY = 14,
+	RHYTHMDB_PROP_TITLE_SORT_KEY = 19,
 	RHYTHMDB_PROP_GENRE_SORT_KEY,
 	RHYTHMDB_PROP_ARTIST_SORT_KEY,
 	RHYTHMDB_PROP_ALBUM_SORT_KEY,
@@ -135,7 +140,8 @@ typedef struct
 	/* signals */
 	void	(*entry_added)		(RhythmDBEntry *entry);
 	void	(*entry_restored)	(RhythmDBEntry *entry);
-	void	(*entry_changed)	(RhythmDBEntry *entry);
+	void	(*entry_changed)	(RhythmDBEntry *entry, RhythmDBPropType prop,
+					 const GValue *old, const GValue *new);
 	void	(*entry_deleted)	(RhythmDBEntry *entry);
 	void	(*load_complete)	(RhythmDBEntry *entry);
 	void	(*legacy_load_complete)	(RhythmDBEntry *entry);
@@ -150,7 +156,7 @@ typedef struct
 						 const char *uri);
 
 	void		(*impl_entry_set)	(RhythmDB *db, RhythmDBEntry *entry,
-						 guint propid, GValue *value);
+						 guint propid, const GValue *value);
 
 	void		(*impl_entry_get)	(RhythmDB *db, RhythmDBEntry *entry,
 						 guint propid, GValue *value);
@@ -183,6 +189,8 @@ void		rhythmdb_write_lock	(RhythmDB *db);
 void		rhythmdb_read_unlock	(RhythmDB *db);
 void		rhythmdb_write_unlock	(RhythmDB *db);
 
+gboolean	rhythmdb_entry_is_editable (RhythmDB *db, RhythmDBEntry *entry);
+
 RhythmDBEntry *	rhythmdb_entry_new	(RhythmDB *db, RhythmDBEntryType type, const char *uri);
 
 void		rhythmdb_add_uri_async	(RhythmDB *db, const char *uri);
@@ -217,6 +225,9 @@ gboolean	rhythmdb_entry_get_boolean	(RhythmDB *db,
 long		rhythmdb_entry_get_long		(RhythmDB *db,
 						 RhythmDBEntry *entry,
 						 guint property_id);
+guint64		rhythmdb_entry_get_uint64	(RhythmDB *db,
+						 RhythmDBEntry *entry,
+						 guint property_id);
 int		rhythmdb_entry_get_int		(RhythmDB *db,
 						 RhythmDBEntry *entry,
 						 guint property_id);
