@@ -2,7 +2,7 @@
  *  arch-tag: Implementation of main Rhythmbox shell
  *
  *  Copyright (C) 2002, 2003 Jorn Baayen
- *  Copyright (C) 2003 Colin Walters <walters@gnome.org>
+ *  Copyright (C) 2003, 2004 Colin Walters <walters@gnome.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -89,6 +89,14 @@
 static void rb_shell_class_init (RBShellClass *klass);
 static void rb_shell_init (RBShell *shell);
 static void rb_shell_finalize (GObject *object);
+static void rb_shell_set_property (GObject *object,
+				   guint prop_id,
+				   const GValue *value,
+				   GParamSpec *pspec);
+static void rb_shell_get_property (GObject *object,
+				   guint prop_id,
+				   GValue *value,
+				   GParamSpec *pspec);
 static void rb_shell_corba_quit (PortableServer_Servant _servant,
                                  CORBA_Environment *ev);
 static void rb_shell_corba_handle_file (PortableServer_Servant _servant,
@@ -119,6 +127,10 @@ static void rb_shell_corba_set_playing_time (PortableServer_Servant _servant,
 static Bonobo_PropertyBag rb_shell_corba_get_player_properties (PortableServer_Servant _servant, CORBA_Environment *ev);
 static void rb_shell_db_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
 				    RBShell *shell);
+static void rb_shell_db_changed_prop_cb (RhythmDB *db, RhythmDBEntry *entry,
+					 RhythmDBPropType prop, const GValue *old,
+					 const GValue *new, RBShell *shell);
+
 
 static gboolean rb_shell_window_state_cb (GtkWidget *widget,
 					  GdkEvent *event,
@@ -202,7 +214,6 @@ static void rb_shell_view_smalldisplay_changed_cb (BonoboUIComponent *component,
 						 const char *state,
 						 RBShell *shell);
 static void rb_shell_load_complete_cb (RhythmDB *db, RBShell *shell);
-static void rb_shell_legacy_load_complete_cb (RhythmDB *db, RBShell *shell);
 static void rb_shell_sync_sourcelist_visibility (RBShell *shell);
 static void rb_shell_sync_smalldisplay (RBShell *shell);
 static void sourcelist_visibility_changed_cb (GConfClient *client,
@@ -243,7 +254,21 @@ static void session_die_cb (GnomeClient *client, RBShell *shell);
 static void rb_shell_session_init (RBShell *shell);
 
 
-static const GtkTargetEntry target_table[] = { { "text/uri-list", 0,0 } };
+static const GtkTargetEntry target_table[] = { { "text/uri-list", 0,0 },
+                                               { "text/x-rhythmbox-album", 0, 0 },
+                                               { "text/x-rhythmbox-artist", 0, 0 },
+                                               { "text/x-rhythmbox-genre", 0, 0 },
+					       { "application/x-rhythmbox-source", 0, 0 }};
+
+enum
+{
+	PROP_NONE,
+	PROP_ARGC,
+	PROP_ARGV,
+	PROP_NO_REGISTRATION,
+	PROP_DRY_RUN,
+	PROP_RHYTHMDB_FILE,
+};
 
 #define CMD_PATH_VIEW_SMALLDISPLAY "/commands/ToggleSmallDisplay"
 #define CMD_PATH_VIEW_SOURCELIST   "/commands/ShowSourceList"
@@ -277,7 +302,11 @@ struct RBShellPrivate
 	gboolean db_dirty;
 	guint async_state_save_id;
 
-	gboolean load_complete;
+	int argc;
+	char **argv;
+	gboolean no_registration;
+	gboolean dry_run;
+	char *rhythmdb_file;
 
 	RhythmDB *db;
 
@@ -372,6 +401,8 @@ rb_shell_class_init (RBShellClass *klass)
 
         parent_class = g_type_class_peek_parent (klass);
 
+	object_class->set_property = rb_shell_set_property;
+	object_class->get_property = rb_shell_get_property;
         object_class->finalize = rb_shell_finalize;
 
 	epv->quit         = rb_shell_corba_quit;
@@ -386,6 +417,45 @@ rb_shell_class_init (RBShellClass *klass)
 	epv->getPlayingTime = rb_shell_corba_get_playing_time;
 	epv->setPlayingTime = rb_shell_corba_set_playing_time;
 	epv->getPlayerProperties = rb_shell_corba_get_player_properties;
+
+	g_object_class_install_property (object_class,
+					 PROP_ARGC,
+					 g_param_spec_int ("argc", 
+							   "argc", 
+							   "Argument count", 
+							   0, 128,
+							   0, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+					 PROP_ARGV,
+					 g_param_spec_pointer ("argv", 
+							       "argv", 
+							       "Arguments", 
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+					 PROP_NO_REGISTRATION,
+					 g_param_spec_boolean ("no-registration", 
+							       "no-registration", 
+							       "Whether or not to register", 
+							       FALSE,
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+					 PROP_DRY_RUN,
+					 g_param_spec_boolean ("dry-run", 
+							       "dry-run", 
+							       "Whether or not this is a dry run", 
+							       FALSE,
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+					 PROP_RHYTHMDB_FILE,
+					 g_param_spec_string ("rhythmdb-file", 
+							      "rhythmdb-file", 
+							      "The RhythmDB file to use", 
+							      "rhythmdb.xml",
+							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
@@ -409,16 +479,75 @@ rb_shell_init (RBShell *shell)
 
 }
 
+static void
+rb_shell_set_property (GObject *object,
+		       guint prop_id,
+		       const GValue *value,
+		       GParamSpec *pspec)
+{
+	RBShell *shell = RB_SHELL (object);
+
+	switch (prop_id)
+	{
+	case PROP_ARGC:
+		shell->priv->argc = g_value_get_int (value);
+		break;
+	case PROP_ARGV:
+		shell->priv->argv = g_value_get_pointer (value);
+		break;
+	case PROP_NO_REGISTRATION:
+		shell->priv->no_registration = g_value_get_boolean (value);
+		break;
+	case PROP_DRY_RUN:
+		shell->priv->dry_run = g_value_get_boolean (value);
+		if (shell->priv->dry_run)
+			shell->priv->no_registration = TRUE;			
+		break;
+	case PROP_RHYTHMDB_FILE:
+		shell->priv->rhythmdb_file = g_value_dup_string (value);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+rb_shell_get_property (GObject *object,
+		       guint prop_id,
+		       GValue *value,
+		       GParamSpec *pspec)
+{
+	RBShell *shell = RB_SHELL (object);
+
+	switch (prop_id)
+	{
+	case PROP_ARGC:
+		g_value_set_int (value, shell->priv->argc);
+		break;
+	case PROP_ARGV:
+		g_value_set_pointer (value, shell->priv->argv);
+		break;
+	case PROP_NO_REGISTRATION:
+		g_value_set_boolean (value, shell->priv->no_registration);
+		break;
+	case PROP_DRY_RUN:
+		g_value_set_boolean (value, shell->priv->dry_run);
+		break;
+	case PROP_RHYTHMDB_FILE:
+		g_value_set_string (value, shell->priv->rhythmdb_file);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 static gboolean
 rb_shell_sync_state (RBShell *shell)
 {
-	if (g_object_get_data (G_OBJECT (shell), "rb-shell-dry-run"))
+	if (shell->priv->dry_run)
 		return FALSE;
-
-	if (!shell->priv->load_complete) {
-		rb_debug ("load incomplete, not syncing state");
-		return FALSE;
-	}
 	
 	rb_debug ("saving playlists");
 	rb_playlist_manager_save_playlists (shell->priv->playlist_manager);
@@ -448,6 +577,14 @@ rb_shell_db_changed_cb (RhythmDB *db, RhythmDBEntry *entry,
 	shell->priv->db_dirty = TRUE;
 }
 
+static void
+rb_shell_db_changed_prop_cb (RhythmDB *db, RhythmDBEntry *entry,
+			     RhythmDBPropType prop, const GValue *old, const GValue *new,
+			     RBShell *shell)
+{
+	rb_shell_db_changed_cb (db, entry, shell);
+}
+
 static void
 rb_shell_finalize (GObject *object)
 {
@@ -488,6 +625,7 @@ rb_shell_finalize (GObject *object)
 	if (shell->priv->prefs != NULL)
 		gtk_widget_destroy (shell->priv->prefs);
 	
+	g_free (shell->priv->rhythmdb_file);
 	g_free (shell->priv);
 
 	g_object_unref (G_OBJECT (rb_file_monitor_get ()));
@@ -499,11 +637,14 @@ rb_shell_finalize (GObject *object)
 }
 
 RBShell *
-rb_shell_new (void)
+rb_shell_new (int argc, char **argv, gboolean no_registration, gboolean dry_run,
+	      char *rhythmdb)
 {
 	RBShell *s;
 
-	s = g_object_new (RB_TYPE_SHELL, NULL);
+	s = g_object_new (RB_TYPE_SHELL, "argc", argc, "argv", argv,
+			  "no-registration", no_registration,
+			  "dry-run", dry_run, "rhythmdb-file", rhythmdb, NULL);
 
 	return s;
 }
@@ -617,10 +758,11 @@ rb_shell_corba_handle_file (PortableServer_Servant _servant,
 {
 	RBShell *shell = RB_SHELL (bonobo_object (_servant));
 	RBPlaylist *parser;
+	GnomeVFSURI *vfsuri;
 
 	rb_debug ("handling uri: %s", uri);
 
-	GnomeVFSURI *vfsuri = gnome_vfs_uri_new (uri);
+	vfsuri = gnome_vfs_uri_new (uri);
 	if (!vfsuri) {
 		rb_error_dialog (_("Unable to parse URI \"%s\"\n"), uri);
 		return;
@@ -709,9 +851,9 @@ get_song_info_from_player (RBShell *shell)
 	song_info->path = CORBA_string_dup (rhythmdb_entry_get_string (db, entry, RHYTHMDB_PROP_LOCATION));
 	song_info->track_number = rhythmdb_entry_get_int (db, entry, RHYTHMDB_PROP_TRACK_NUMBER);
 	song_info->duration = rhythmdb_entry_get_long (db, entry, RHYTHMDB_PROP_DURATION);
-	song_info->bitrate = rhythmdb_entry_get_int (db, entry, RHYTHMDB_PROP_QUALITY);
-	song_info->filesize = rhythmdb_entry_get_long (db, entry, RHYTHMDB_PROP_FILE_SIZE);
-	song_info->rating = rhythmdb_entry_get_int (db, entry, RHYTHMDB_PROP_RATING);
+	song_info->bitrate = rhythmdb_entry_get_int (db, entry, RHYTHMDB_PROP_BITRATE);
+	song_info->filesize = rhythmdb_entry_get_uint64 (db, entry, RHYTHMDB_PROP_FILE_SIZE);
+	song_info->rating = rhythmdb_entry_get_double (db, entry, RHYTHMDB_PROP_RATING);
 	song_info->play_count = rhythmdb_entry_get_int (db, entry, RHYTHMDB_PROP_PLAY_COUNT);
 	song_info->last_played = rhythmdb_entry_get_long (db, entry, RHYTHMDB_PROP_LAST_PLAYED);
 	rhythmdb_read_unlock (db);
@@ -924,7 +1066,7 @@ rb_shell_entry_changed_cb (GObject *object, GParamSpec *pspec, RBShell *shell)
 {
 	GNOME_Rhythmbox_SongInfo *song_info;
 	BonoboArg *arg;
-
+	
 	g_assert (strcmp (pspec->name, "playing-entry") == 0);
 	song_info = get_song_info_from_player (shell);
 	arg = bonobo_arg_new (TC_GNOME_Rhythmbox_SongInfo);
@@ -947,7 +1089,7 @@ rb_shell_entry_changed_cb (GObject *object, GParamSpec *pspec, RBShell *shell)
        		dashboard_send_raw_cluepacket (cluepacket);
        		g_free (cluepacket);
 	}
-#endif //WITH_DASHBOARD
+#endif /* WITH_DASHBOARD */
 }
 
 void
@@ -1006,20 +1148,26 @@ rb_shell_construct (RBShell *shell)
 	/* Initialize the database */
 	rb_debug ("creating database object");
 	{
-		char *fname = g_build_filename (rb_dot_dir (), "rhythmdb.xml", NULL);
+		char *pathname;
 
-		rhythmdb_exists = g_file_test (fname, G_FILE_TEST_EXISTS);
+		if (shell->priv->rhythmdb_file)
+			pathname = g_strdup (shell->priv->rhythmdb_file);
+		else
+			pathname = g_build_filename (rb_dot_dir (), "rhythmdb.xml", NULL);
+
+		rhythmdb_exists = g_file_test (pathname, G_FILE_TEST_EXISTS);
 		
-		shell->priv->db = rhythmdb_tree_new (fname);
-		g_free (fname);
+		shell->priv->db = rhythmdb_tree_new (pathname);
+		g_free (pathname);
+
+		if (shell->priv->dry_run)
+			g_object_set (G_OBJECT (shell->priv->db), "dry-run", TRUE, NULL);
 
 		if (rhythmdb_exists) {
 			g_signal_connect_object (G_OBJECT (shell->priv->db), "load-complete",
 						 G_CALLBACK (rb_shell_load_complete_cb), shell,
 						 0);
 			rhythmdb_load (shell->priv->db);
-		} else {
-			shell->priv->load_complete = TRUE;
 		}
 	}
 	g_signal_connect_object (G_OBJECT (shell->priv->db),
@@ -1028,21 +1176,13 @@ rb_shell_construct (RBShell *shell)
 				 shell, 0);
 	g_signal_connect_object (G_OBJECT (shell->priv->db),
 				 "entry_changed",
-				 G_CALLBACK (rb_shell_db_changed_cb),
+				 G_CALLBACK (rb_shell_db_changed_prop_cb),
 				 shell, 0);
 	g_signal_connect_object (G_OBJECT (shell->priv->db),
 				 "entry_deleted",
 				 G_CALLBACK (rb_shell_db_changed_cb),
 				 shell, 0);
 
-	if (!rhythmdb_exists && eel_gconf_get_boolean (CONF_FIRST_TIME)) {
-		rb_debug ("loading legacy library db");
-		g_signal_connect_object (G_OBJECT (shell->priv->db), "legacy-load-complete",
-					 G_CALLBACK (rb_shell_legacy_load_complete_cb), shell,
-					 0);
-		rhythmdb_load_legacy (shell->priv->db);
-	}
-
 	rb_debug ("shell: setting up tray icon");
 	tray_deleted_cb (NULL, NULL, shell);
 
@@ -1159,8 +1299,6 @@ rb_shell_construct (RBShell *shell)
 	shell->priv->show_db_errors = FALSE;
 	gtk_widget_hide (shell->priv->load_error_dialog);
 
-	shell->priv->supported_media_extensions = monkey_media_get_supported_filename_extensions ();
-
 	g_signal_connect (G_OBJECT (shell->priv->db), "error",
 			  G_CALLBACK (rb_shell_db_error_cb), shell);
 
@@ -1169,7 +1307,7 @@ rb_shell_construct (RBShell *shell)
 
 	/* initialize sources */
 	rb_debug ("shell: creating library source");
-	shell->priv->library_source = rb_library_source_new (shell->priv->db);
+	shell->priv->library_source = rb_library_source_new (shell->priv->db, shell->priv->ui_component);
 	rb_shell_append_source (shell, shell->priv->library_source);
 
 	rb_debug ("shell: creating iradio source");
@@ -1218,8 +1356,8 @@ rb_shell_construct (RBShell *shell)
 		rb_debug ("No AudioCD device is available!");
 #endif
 	
-	if (!g_object_get_data (G_OBJECT (shell), "rb-shell-no-registration")
-	    && !g_object_get_data (G_OBJECT (shell), "rb-shell-dry-run")) {
+	if (!shell->priv->no_registration) {
+		rb_debug ("Registering with Bonobo Activation...");
 		/* register with CORBA */
 		CORBA_exception_init (&ev);
 		
@@ -1401,37 +1539,15 @@ rb_shell_db_error_cb (RhythmDB *db,
 		      const char *uri, const char *msg,
 		      RBShell *shell)
 {
-	GList *tem;
-	GnomeVFSURI *vfsuri;
-	gchar *basename;
-	gssize baselen;
-
 	rb_debug ("got db error, showing: %s",
 		  shell->priv->show_db_errors ? "TRUE" : "FALSE");
 	
 	if (!shell->priv->show_db_errors)
 		return;
 
-	vfsuri = gnome_vfs_uri_new (uri);
-	basename = gnome_vfs_uri_extract_short_name (vfsuri);
-	baselen = strlen (basename);
-	
-	for (tem = shell->priv->supported_media_extensions; tem != NULL; tem = g_list_next (tem)) {
-		gssize extlen = strlen (tem->data);
-		if (extlen >= baselen)
-			continue;
-		if (!strncmp (basename + (baselen - extlen), tem->data, extlen)) {
-			rb_debug ("\"%s\" matches \"%s\"", basename, tem->data);
-			rb_load_failure_dialog_add (RB_LOAD_FAILURE_DIALOG (shell->priv->load_error_dialog),
-						    uri, msg);
-			gtk_widget_show (GTK_WIDGET (shell->priv->load_error_dialog));
-			goto out;
-		}
-	}
-	rb_debug ("\"%s\" has unknown extension, ignoring", basename);
-out:
-	g_free (basename);
-	gnome_vfs_uri_unref (vfsuri);
+	rb_load_failure_dialog_add (RB_LOAD_FAILURE_DIALOG (shell->priv->load_error_dialog),
+				    uri, msg);
+	gtk_widget_show (GTK_WIDGET (shell->priv->load_error_dialog));
 }
 
 static void
@@ -1655,7 +1771,7 @@ rb_shell_cmd_about (BonoboUIComponent *component,
 		    const char *verbname)
 {
 	const char **tem;
-	char *comment;
+	GString *comment;
 	static GtkWidget *about = NULL;
 	GdkPixbuf *pixbuf = NULL;
 
@@ -1693,9 +1809,12 @@ rb_shell_cmd_about (BonoboUIComponent *component,
 		;
 	*tem = _("Contributors:");
 
+	comment = g_string_new (_("Music management and playback software for GNOME."));
+#ifdef WITH_MONKEYMEDIA
 	{
 		const char *backend;
 		GString *formats = g_string_new ("");
+
 #ifdef HAVE_GSTREAMER
 		backend = "GStreamer";
 #else
@@ -1713,19 +1832,20 @@ rb_shell_cmd_about (BonoboUIComponent *component,
 #ifdef HAVE_MP4
 		g_string_append (formats, " MPEG-4");
 #endif
-
-		comment = g_strdup_printf (_("Music management and playback software for GNOME.\nAudio backend: %s\nAudio formats:%s\n"), backend, formats->str);
+		g_string_append_printf (comment, "\nAudio playback: %s\nAudio formats:%s\n",
+					backend, formats->str);
 
 		g_string_free (formats, TRUE);
 	}
+#endif
 	about = gnome_about_new ("Rhythmbox", VERSION,
-				 "Copyright \xc2\xa9 2002, 2003 Jorn Baayen, Colin Walters",
-				 comment,
+				 "Copyright \xc2\xa9 2003, 2004 Colin Walters\nCopyright \xc2\xa9 2002, 2003 Jorn Baayen",
+				 comment->str,
 				 (const char **) authors,
 				 (const char **) documenters,
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				 pixbuf);
-	g_free (comment);
+	g_string_free (comment, TRUE);
 	gtk_window_set_transient_for (GTK_WINDOW (about), GTK_WINDOW (shell->priv->window));
 
 	g_object_add_weak_pointer (G_OBJECT (about),
@@ -1778,12 +1898,8 @@ ask_file_response_cb (GtkDialog *dialog,
 		      int response_id,
 		      RBShell *shell)
 {
-	char **files, **filecur, *stored;
-
-	if (response_id != GTK_RESPONSE_OK) {
-		gtk_widget_destroy (GTK_WIDGET (dialog));
-		return;
-	}
+#ifndef HAVE_GTK_2_3
+	char **files, **filecur;
 
 	files = gtk_file_selection_get_selections (GTK_FILE_SELECTION (dialog));
 
@@ -1795,13 +1911,22 @@ ask_file_response_cb (GtkDialog *dialog,
 	filecur = files;
 
 	if (*filecur != NULL) {
-		char *tmp;
+		char *tmp, *tmp2;
+		GnomeVFSResult result;
+		GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
 
-		stored = g_path_get_dirname (*filecur);
-		tmp = g_strconcat (stored, "/", NULL);
-		eel_gconf_set_string (CONF_STATE_ADD_DIR, tmp);
-		g_free (tmp);
-		g_free (stored);
+		if ((result = gnome_vfs_get_file_info (*filecur, info, GNOME_VFS_FILE_INFO_DEFAULT| GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE)) == GNOME_VFS_OK) {
+			if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) 
+				tmp = g_strconcat (*filecur, "/", NULL);
+			else {
+				tmp2 = g_path_get_dirname (*filecur);
+				tmp = g_strconcat (tmp2, "/", NULL);
+				g_free (tmp2);
+			}
+			eel_gconf_set_string (CONF_STATE_ADD_DIR, tmp);
+			g_free (tmp);
+		}
+		gnome_vfs_file_info_unref (info);
 	}
 
 	shell->priv->show_db_errors = TRUE;
@@ -1816,6 +1941,33 @@ ask_file_response_cb (GtkDialog *dialog,
 	}
 
 	g_strfreev (files);
+
+#else
+	char *current_dir = NULL;
+	GSList *uri_list = NULL, *uris = NULL;
+
+	if (response_id != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (GTK_WIDGET (dialog));
+		return;
+	}
+
+	uri_list = gtk_file_chooser_get_uris (GTK_FILE_CHOOSER (dialog));
+
+	if (uri_list == NULL) {
+		current_dir = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (dialog));
+		uri_list = g_slist_append (uri_list, current_dir);
+	}
+	
+	shell->priv->show_db_errors = TRUE;
+
+	for (uris = uri_list; uris; uris = uris->next) {
+		if (g_utf8_validate ((char *)uris->data, -1, NULL)) 
+			rhythmdb_add_uri_async (shell->priv->db, (char *)uris->data);
+	}
+	g_slist_free (uri_list);
+	g_free (current_dir);
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+#endif
 }
 
 static void
@@ -1835,7 +1987,7 @@ rb_shell_cmd_add_to_library (BonoboUIComponent *component,
 	g_signal_connect (G_OBJECT (dialog),
 			  "response",
 			  G_CALLBACK (ask_file_response_cb),
-			  shell);
+				shell);
 }
 
 static void
@@ -1896,17 +2048,9 @@ rb_shell_load_complete_cb (RhythmDB *db, RBShell *shell)
 	rb_debug ("load complete");
 	GDK_THREADS_ENTER ();
 	rb_playlist_manager_load_playlists (shell->priv->playlist_manager);
-	shell->priv->load_complete = TRUE;
 	GDK_THREADS_LEAVE ();
 }
 
-static void
-rb_shell_legacy_load_complete_cb (RhythmDB *db, RBShell *shell)
-{
-	rb_debug ("legacy load complete");
-	rb_playlist_manager_load_legacy_playlists (shell->priv->playlist_manager);
-}
-
 static void
 rb_shell_sync_sourcelist_visibility (RBShell *shell)
 {
@@ -2040,48 +2184,20 @@ sourcelist_drag_received_cb (RBSourceList *sourcelist,
 			     GtkSelectionData *data,
 			     RBShell *shell)
 {
-	if (source == NULL) 
-		source = rb_playlist_manager_new_playlist (shell->priv->playlist_manager, 
-							   FALSE);
-	
-	if (source != NULL) {
-		rb_source_receive_drag (source, data);
-		return;
-	} 
+        gboolean smart;
 
+        if (source == NULL) {
+		char *datastr = g_strndup (data->data, data->length);
+                smart = data->type != gdk_atom_intern ("text/uri-list", TRUE);
+                source = rb_playlist_manager_new_playlist (shell->priv->playlist_manager, 
+							   datastr, smart);
+		g_free (datastr);
+        }
 
-/* 	if (data->type == gdk_atom_intern (RB_LIBRARY_DND_NODE_ID_TYPE, TRUE)) { */
-/* 		long id; */
-/* 		RBNode *node; */
-/* 		RBPlaylistSource *playlist; */
+        if (source != NULL) {
+                rb_source_receive_drag (source, data);
+        }
 
-
-/* 		id = atol (data->data); */
-/* 		rb_debug ("got node id %d", id); */
-/* 		node = rb_node_db_get_node_from_id (rb_library_get_node_db (shell->priv->library), id); */
-
-/* 		if (node == NULL) */
-/* 			return; */
-			
-		
-/* 		playlist = create_playlist_with_name (shell, rb_node_get_property_string (node, RB_NODE_PROP_NAME)); */
-/* 		if (playlist == NULL) */
-/* 			return; */
-
-/* 		rb_library_handle_songs (shell->priv->library, */
-/* 					 node, */
-/* 					 (GFunc) handle_songs_func, */
-/* 					 playlist); */
-
-/* 		shell->priv->playlists = g_list_append (shell->priv->playlists, playlist); */
-/* 		rb_shell_append_source (shell, RB_SOURCE (playlist)); */
-/* 	} else { */
-/* 		GList *list; */
-
-/* 		rb_debug ("got vfs data, len: %d", data->length); */
-/* 		list = gnome_vfs_uri_list_parse (data->data); */
-/* 		create_playlist (shell, CREATE_PLAYLIST_WITH_URI_LIST, list); */
-/* 	} */
 }
 
 static void
@@ -2206,7 +2322,7 @@ save_yourself_cb (GnomeClient *client,
                   RBShell *shell)
 {
         rb_debug ("session save yourself");
-	rb_shell_sync_state (shell);
+        rb_shell_sync_state (shell);
         return TRUE;
 }
 
