@@ -35,6 +35,7 @@
 #include "rb-search-entry.h"
 #include "rb-debug.h"
 #include "eel-gconf-extensions.h"
+#include "rb-play-order.h"
 
 static GObject* rb_statusbar_construct (GType type,
 					guint n_construct_properties,
@@ -56,6 +57,7 @@ static void rb_statusbar_state_changed_cb (GConfClient *client,
 					   GConfEntry *entry,
 					   RBStatusbar *statusbar);
 
+static void rb_statusbar_sync_status (RBStatusbar *status);
 static gboolean poll_status (RBStatusbar *status);
 static void rb_statusbar_toggle_changed_cb (GtkToggleButton *toggle,
 					    RBStatusbar *statusbar);
@@ -98,6 +100,7 @@ struct RBStatusbarPrivate
 
 	gboolean entry_view_busy;
 	gboolean library_busy;
+	gboolean buffering;
 	guint idle_tick_id;
 };
 
@@ -205,13 +208,17 @@ rb_statusbar_construct (GType                  type,
 	statusbar = RB_STATUSBAR (object);
 
 	g_signal_connect_swapped (G_OBJECT (statusbar->priv->player),
-				  "notify::shuffle", 
+				  "notify::play-order", 
 				  G_CALLBACK (rb_statusbar_sync_state), 
 				  statusbar);
 	g_signal_connect_swapped (G_OBJECT (statusbar->priv->player),
 				  "notify::repeat", 
 				  G_CALLBACK (rb_statusbar_sync_state), 
 				  statusbar);
+	g_signal_connect_swapped (G_OBJECT (statusbar->priv->player),
+				  "notify::buffering", 
+				  G_CALLBACK (rb_statusbar_sync_status), 
+				  statusbar);
 
 	return object;
 }
@@ -399,17 +406,17 @@ status_tick_cb (GtkProgressBar *progress)
 	return TRUE;
 }
 
-static gboolean
-poll_status (RBStatusbar *status)
+static void
+rb_statusbar_sync_status (RBStatusbar *status)
 {
 	char *str;
 	gboolean library_busy_changed;
+	gboolean buffering_changed;
 	gboolean library_is_busy;
+	gboolean buffering;
 	gboolean entry_view_busy = FALSE;
 	gboolean changed = FALSE;
 
-	GDK_THREADS_ENTER ();
-
 	str = rhythmdb_get_status (status->priv->db);
 
 	if (!str && status->priv->selected_source) {
@@ -422,18 +429,32 @@ poll_status (RBStatusbar *status)
 	library_busy_changed = status->priv->library_busy && !library_is_busy;
 	status->priv->library_busy = library_is_busy;	
 
+	g_object_get (G_OBJECT (status->priv->player), "buffering",
+		      &buffering, NULL);
+	buffering_changed = status->priv->buffering && !buffering;
+	status->priv->buffering = buffering;	
+
 	/* Set up the status display */
 	if (status->priv->library_busy) {
 		gtk_label_set_markup (GTK_LABEL (status->priv->status), str);
 		g_free (str);
+		str = NULL;
 		changed = TRUE;
-	} else if (library_busy_changed) {
+	} else if (buffering) {
+		g_free (str);
+		str = g_strdup_printf ("<b>%s</b>",
+				       _("Buffering..."));
+		gtk_label_set_markup (GTK_LABEL (status->priv->status), str);
+		g_free (str);
+		str = NULL;
+	} else if (library_busy_changed || buffering_changed) {
 		rb_statusbar_sync_with_source (status);
 		changed = TRUE;
 	}
+	g_free (str);
 
 	/* Sync the progress bar */
-	if (library_is_busy || entry_view_busy) {
+	if (library_is_busy || entry_view_busy || buffering) {
 		if (status->priv->idle_tick_id == 0) {
 			status->priv->idle_tick_id
 				= g_timeout_add (250, (GSourceFunc) status_tick_cb,
@@ -452,7 +473,17 @@ poll_status (RBStatusbar *status)
 
 	status->priv->status_poll_id = 
 		g_timeout_add (changed ? 350 : 750, (GSourceFunc) poll_status, status);
+}
+
+static gboolean
+poll_status (RBStatusbar *status)
+{
+	GDK_THREADS_ENTER ();
+
+	rb_statusbar_sync_status (status);
+
 	GDK_THREADS_LEAVE ();
+
 	return FALSE;
 }
 
@@ -483,19 +514,30 @@ void
 rb_statusbar_sync_state (RBStatusbar *statusbar)
 {
 	gboolean hidden;
-	gboolean shuffle;
+	const gchar *play_order;
 	gboolean repeat;
+	gboolean is_linear, is_shuffle;
 
 	rb_debug ("syncing state");
 
 	g_object_get (G_OBJECT (statusbar->priv->player), 
-		      "shuffle", &shuffle,
+		      "play-order", &play_order,
 		      "repeat", &repeat,
 		      NULL);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (statusbar->priv->shuffle),
-				      shuffle);
+
+	is_linear = (strcmp (play_order, "linear") == 0);
+	is_shuffle = (strcmp (play_order, "shuffle") == 0);
+	if (is_linear || is_shuffle) {
+		/* setting active sets the play order to one of shuffle or random */
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (statusbar->priv->shuffle),
+					      !is_linear);
+	}
+	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (statusbar->priv->shuffle),
+					    !is_linear && !is_shuffle);
+	
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (statusbar->priv->repeat),
 				      repeat);
+
 	
 	hidden = eel_gconf_get_boolean (CONF_UI_STATUSBAR_HIDDEN);
 	if (hidden)
@@ -527,7 +569,7 @@ rb_statusbar_toggle_changed_cb (GtkToggleButton *toggle,
 	rb_debug ("toggle changed");
 
 	if (toggle == GTK_TOGGLE_BUTTON (statusbar->priv->shuffle)) {
-		rb_shell_player_set_shuffle (player, gtk_toggle_button_get_active (toggle));
+		rb_shell_player_set_play_order (player, gtk_toggle_button_get_active (toggle) ? "shuffle" : "linear");
 	} else if (toggle == GTK_TOGGLE_BUTTON (statusbar->priv->repeat)) {
 		rb_shell_player_set_repeat (player, gtk_toggle_button_get_active (toggle));
 	} else {
