@@ -2,7 +2,7 @@
  *  arch-tag: Implementation of local file source object
  *
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
- *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
+ *  Copyright (C) 2003,2004 Colin Walters <walters@verbum.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -73,6 +73,15 @@ static void rb_library_source_get_property (GObject *object,
 			                  guint prop_id,
 			                  GValue *value,
 			                  GParamSpec *pspec);
+static void rb_library_source_cmd_choose_genre (BonoboUIComponent *component,
+						 RBLibrarySource *source,
+						 const char *verbname);
+static void rb_library_source_cmd_choose_artist (BonoboUIComponent *component,
+						 RBLibrarySource *source,
+						 const char *verbname);
+static void rb_library_source_cmd_choose_album (BonoboUIComponent *component,
+						RBLibrarySource *source,
+						const char *verbname);
 static void rb_library_source_handle_genre_selection (RBLibrarySource *libsource, GList *genres);
 static void rb_library_source_handle_artist_selection (RBLibrarySource *libsource, GList *artists);
 static void rb_library_source_handle_album_selection (RBLibrarySource *libsource, GList *albums);
@@ -91,15 +100,6 @@ static void songs_view_sort_order_changed_cb (RBEntryView *view, RBLibrarySource
 static void paned_size_allocate_cb (GtkWidget *widget,
 				    GtkAllocation *allocation,
 		                    RBLibrarySource *source);
-/* static void rb_library_source_drop_cb (GtkWidget        *widget, */
-/* 				       GdkDragContext   *context, */
-/* 				       gint              x, */
-/* 				       gint              y, */
-/* 				       GtkSelectionData *data, */
-/* 				       guint             info, */
-/* 				       guint             time, */
-/* 				       gpointer          user_data); */
-
 static void rb_library_source_state_pref_changed (GConfClient *client,
 						 guint cnxn_id,
 						 GConfEntry *entry,
@@ -131,7 +131,6 @@ static const char * impl_get_album (RBSource *player);
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
 static gboolean impl_show_popup (RBSource *source);
 static void rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype);
-static void query_complete_cb (RhythmDBQueryModel *model, RBLibrarySource *source);
 
 void rb_library_source_browser_views_activated_cb (GtkWidget *widget,
 						 RBLibrarySource *source);
@@ -190,6 +189,14 @@ struct RBLibrarySourcePrivate
 	GSList *browser_views_group;
 };
 
+static BonoboUIVerb rb_library_source_verbs[] =
+{
+	BONOBO_UI_VERB ("SLChooseGenre",(BonoboUIVerbFn) rb_library_source_cmd_choose_genre),
+	BONOBO_UI_VERB ("SLChooseArtist",(BonoboUIVerbFn) rb_library_source_cmd_choose_artist),
+	BONOBO_UI_VERB ("SLChooseAlbum", (BonoboUIVerbFn) rb_library_source_cmd_choose_album),
+	BONOBO_UI_VERB_END
+};
+
 enum
 {
 	PROP_0,
@@ -199,9 +206,6 @@ enum
 
 static GObjectClass *parent_class = NULL;
 
-/* dnd */
-static const GtkTargetEntry target_uri [] = { { "text/uri-list", 0, 0 } };
-
 GType
 rb_library_source_get_type (void)
 {
@@ -411,6 +415,7 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 	RBLibrarySource *source;
 	RBLibrarySourceClass *klass;
 	GObjectClass *parent_class;  
+	BonoboUIComponent *component;
 
 	klass = RB_LIBRARY_SOURCE_CLASS (g_type_class_peek (type));
 
@@ -418,6 +423,10 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 	source = RB_LIBRARY_SOURCE (parent_class->constructor (type, n_construct_properties,
 							       construct_properties));
 
+	g_object_get (G_OBJECT (source), "component", &component, NULL);
+	bonobo_ui_component_add_verb_list_with_data (component, rb_library_source_verbs,
+						     source);
+
 	source->priv->paned = gtk_vpaned_new ();
 
 	source->priv->browser = gtk_hbox_new (TRUE, 5);
@@ -488,27 +497,6 @@ rb_library_source_constructor (GType type, guint n_construct_properties,
 
 	rb_library_source_do_query (source, RB_LIBRARY_QUERY_TYPE_ALL);
 
-	/* Drag'n'Drop for songs view */
-/* 	g_signal_connect (G_OBJECT (source->priv->songs), "drag_data_received", */
-/* 			  G_CALLBACK (rb_library_source_drop_cb), source); */
-/* 	gtk_drag_dest_set (GTK_WIDGET (source->priv->songs), GTK_DEST_DEFAULT_ALL, */
-/* 			   target_uri, 1, GDK_ACTION_COPY); */
-/* 	rb_node_view_enable_drag_source (source->priv->songs, target_uri, 1); */
-
-/* 	/\* Drag'n'Drop for albums view *\/ */
-/* 	g_signal_connect (G_OBJECT (source->priv->albums), "drag_data_received", */
-/* 			  G_CALLBACK (rb_library_source_drop_cb), source); */
-/* 	gtk_drag_dest_set (GTK_WIDGET (source->priv->albums), GTK_DEST_DEFAULT_ALL, */
-/* 			   target_uri, 1, GDK_ACTION_COPY); */
-/* 	rb_node_view_enable_drag_source (source->priv->albums, target_id, 1); */
-
-/* 	/\* Drag'n'Drop for artists view *\/ */
-/* 	g_signal_connect (G_OBJECT (source->priv->artists), "drag_data_received", */
-/* 			  G_CALLBACK (rb_library_source_drop_cb), source); */
-/* 	gtk_drag_dest_set (GTK_WIDGET (source->priv->artists), GTK_DEST_DEFAULT_ALL, */
-/* 			   target_uri, 1, GDK_ACTION_COPY); */
-/* 	rb_node_view_enable_drag_source (source->priv->artists, target_id, 1); */
-
 	/* this gets emitted when the paned thingie is moved */
 	g_signal_connect (G_OBJECT (source->priv->songs),
 			  "size_allocate",
@@ -574,35 +562,19 @@ rb_library_source_get_property (GObject *object,
 }
 
 RBSource *
-rb_library_source_new (RhythmDB *db)
+rb_library_source_new (RhythmDB *db, BonoboUIComponent *component)
 {
 	RBSource *source;
 
 	source = RB_SOURCE (g_object_new (RB_TYPE_LIBRARY_SOURCE,
 					  "name", _("Library"),
+					  "component", component,
 					  "db", db,
 					  NULL));
 
 	return source;
 }
 
-/* static RBNode * */
-/* ensure_node_selection (RBEntryView *source, */
-/* 		       RBNode *all_node, */
-/* 		       gboolean *changing_flag) */
-/* { */
-/* 	GList *selection = rb_node_view_get_selection (source); */
-
-/* 	if (selection == NULL) { */
-/* 		*changing_flag = TRUE; */
-/* 		rb_node_view_select_node (source, all_node); */
-/* 		*changing_flag = FALSE; */
-/* 		selection = rb_node_view_get_selection (source); */
-/* 	} */
-
-/* 	return selection->data; */
-/* } */
-
 static gboolean
 string_list_equal (GList *a, GList *b)
 {
@@ -639,6 +611,85 @@ string_list_equal (GList *a, GList *b)
 	return ret;
 }
 
+
+
+static void
+rb_library_source_gather_hash_keys (char *key, gpointer unused,
+				    GList **data)
+{
+	*data = g_list_prepend (*data, key);
+}
+
+static GList *
+rb_library_source_gather_properties (RBLibrarySource *source,
+				     RhythmDBPropType prop)
+{
+	GList *selected, *tem;
+	GHashTable *selected_set;
+
+	rhythmdb_read_lock (source->priv->db);
+
+	selected_set = g_hash_table_new (g_str_hash, g_str_equal);
+	selected = rb_entry_view_get_selected_entries (source->priv->songs);
+	for (tem = selected; tem; tem = tem->next) {
+		char *val = g_strdup (rhythmdb_entry_get_string (source->priv->db,
+								 tem->data, prop));
+		g_hash_table_insert (selected_set, val, NULL);
+	}
+
+	rhythmdb_read_unlock (source->priv->db);
+
+	g_list_free (selected);
+	
+	tem = NULL;
+	g_hash_table_foreach (selected_set, (GHFunc) rb_library_source_gather_hash_keys,
+			      &tem);
+	g_hash_table_destroy (selected_set);
+	return tem;
+}
+
+static void
+rb_library_source_cmd_choose_genre (BonoboUIComponent *component,
+				    RBLibrarySource *source,
+				    const char *verbname)
+{
+	GList *props;	
+
+	rb_debug ("choosing genre");
+
+	props = rb_library_source_gather_properties (source, RHYTHMDB_PROP_GENRE);
+	rb_property_view_set_selection (source->priv->genres, props);
+	g_list_free (props);
+}
+
+static void
+rb_library_source_cmd_choose_artist (BonoboUIComponent *component,
+				     RBLibrarySource *source,
+				     const char *verbname)
+{
+	GList *props;	
+
+	rb_debug ("choosing artist");
+
+	props = rb_library_source_gather_properties (source, RHYTHMDB_PROP_ARTIST);
+	rb_property_view_set_selection (source->priv->artists, props);
+	g_list_free (props);
+}
+
+static void
+rb_library_source_cmd_choose_album (BonoboUIComponent *component,
+				    RBLibrarySource *source,
+				    const char *verbname)
+{
+	GList *props;	
+
+	rb_debug ("choosing album");
+
+	props = rb_library_source_gather_properties (source, RHYTHMDB_PROP_ALBUM);
+	rb_property_view_set_selection (source->priv->albums, props);
+	g_list_free (props);
+}
+
 static void
 rb_library_source_sync_query (RBLibrarySource *source)
 {
@@ -745,6 +796,7 @@ static void
 songs_view_sort_order_changed_cb (RBEntryView *view, RBLibrarySource *source)
 {
 	rb_debug ("sort order changed");
+	rb_entry_view_set_resorting (view);
 	rb_library_source_do_query (source, RB_LIBRARY_QUERY_TYPE_SEARCH);
 }
 
@@ -1280,9 +1332,6 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 
 	if (source->priv->active_query) {
 		rb_debug ("killing active query");
-		g_signal_handlers_disconnect_by_func (G_OBJECT (source->priv->active_query),
-						      G_CALLBACK (query_complete_cb),
-						      source);
 		rhythmdb_query_model_cancel (RHYTHMDB_QUERY_MODEL (source->priv->active_query));
 	}
 
@@ -1356,55 +1405,62 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 		g_object_ref (G_OBJECT (source->priv->cached_albums_model));
 	} else {
 		rb_debug ("query is not special");
-		if (qtype < RB_LIBRARY_QUERY_TYPE_GENRE) {
-			rb_debug ("resetting genres view");
-			rb_property_view_reset (source->priv->genres);
-			g_list_foreach (source->priv->selected_genres, (GFunc) g_free, NULL);
-			g_list_free (source->priv->selected_genres);
-			source->priv->selected_genres = NULL;
-		}
-		genre_model = rb_property_view_get_model (source->priv->genres);
-		if (qtype < RB_LIBRARY_QUERY_TYPE_ARTIST) {
-			rb_debug ("resetting artist view");
-			rb_property_view_reset (source->priv->artists);
-			g_list_foreach (source->priv->selected_artists, (GFunc) g_free, NULL);
-			g_list_free (source->priv->selected_artists);
-			source->priv->selected_artists = NULL;
-		}
-		artist_model = rb_property_view_get_model (source->priv->artists);
-		if (qtype < RB_LIBRARY_QUERY_TYPE_ALBUM) {
-			rb_debug ("resetting album view");
-			rb_property_view_reset (source->priv->albums);
-			g_list_foreach (source->priv->selected_albums, (GFunc) g_free, NULL);
-			g_list_free (source->priv->selected_albums);
-			source->priv->selected_albums = NULL;
-		}
-		album_model = rb_property_view_get_model (source->priv->albums);
-
 		source->priv->active_query = source->priv->model = 
 			query_model = rhythmdb_query_model_new_empty (source->priv->db);
-		if (qtype < RB_LIBRARY_QUERY_TYPE_GENRE)
-			g_object_set (G_OBJECT (genre_model), "query-model", query_model, NULL);
-		else
-			g_object_set (G_OBJECT (genre_model), "query-model", NULL, NULL);
-	
-		if (qtype < RB_LIBRARY_QUERY_TYPE_ARTIST)
-			g_object_set (G_OBJECT (artist_model), "query-model", query_model, NULL);
-		else
-			g_object_set (G_OBJECT (artist_model), "query-model", NULL, NULL);
 
-		if (qtype < RB_LIBRARY_QUERY_TYPE_ALBUM)
-			g_object_set (G_OBJECT (album_model), "query-model", query_model, NULL);
-		else
-			g_object_set (G_OBJECT (album_model), "query-model", NULL, NULL);
+		if (source->priv->selected_genres == NULL) {
+			rb_property_view_set_model (source->priv->genres,
+						    source->priv->cached_genres_model);
+			if (source->priv->selected_artists == NULL) {
+				rb_property_view_set_model (source->priv->artists,
+							    source->priv->cached_artists_model);
+				if (source->priv->selected_albums == NULL)
+					rb_property_view_set_model (source->priv->albums,
+								    source->priv->cached_albums_model);
+			}
+		}
+		switch (qtype) {
+		case RB_LIBRARY_QUERY_TYPE_ALL:
+			rb_debug ("resetting genres view");
+			g_list_foreach (source->priv->selected_genres,
+					(GFunc) g_free, NULL);
+			g_list_free (source->priv->selected_genres);
+			source->priv->selected_genres = NULL;
+
+			rb_property_view_reset (source->priv->genres);
+			genre_model = rb_property_view_get_model (source->priv->genres);
+			g_object_set (G_OBJECT (genre_model), "query-model",
+				      query_model, NULL);
+		case RB_LIBRARY_QUERY_TYPE_GENRE:
+			rb_debug ("resetting artist view");
+			g_list_foreach (source->priv->selected_artists,
+					(GFunc) g_free, NULL);
+			g_list_free (source->priv->selected_artists);
+			source->priv->selected_artists = NULL;
+
+			rb_property_view_reset (source->priv->artists);
+			artist_model = rb_property_view_get_model (source->priv->artists);
+			g_object_set (G_OBJECT (artist_model), "query-model",
+				      query_model, NULL);
+		case RB_LIBRARY_QUERY_TYPE_ARTIST:
+			rb_debug ("resetting album view");
+			g_list_foreach (source->priv->selected_albums,
+					(GFunc) g_free, NULL);
+			g_list_free (source->priv->selected_albums);
+			source->priv->selected_albums = NULL;
+
+			rb_property_view_reset (source->priv->albums);
+			album_model = rb_property_view_get_model (source->priv->albums);
+			g_object_set (G_OBJECT (album_model), "query-model",
+				      query_model, NULL);
+		case RB_LIBRARY_QUERY_TYPE_ALBUM:
+		case RB_LIBRARY_QUERY_TYPE_SEARCH:
+			break;
+		}
 	}
 
 	model = GTK_TREE_MODEL (query_model);
 
-	g_signal_connect_object (G_OBJECT (query_model),
-				 "complete", G_CALLBACK (query_complete_cb),
-				 source, 0);
-	
 	rb_debug ("setting empty model");
 	rb_entry_view_set_model (source->priv->songs, RHYTHMDB_QUERY_MODEL (query_model));
 
@@ -1420,61 +1476,3 @@ rb_library_source_do_query (RBLibrarySource *source, RBLibraryQueryType qtype)
 	rb_entry_view_poll_model (source->priv->songs);
 	rb_debug ("done polling");
 }
-
-static void
-query_complete_cb (RhythmDBQueryModel *model, RBLibrarySource *source)
-{
-	RhythmDBPropertyModel *genre_model = rb_property_view_get_model (source->priv->genres);
-	RhythmDBPropertyModel *artist_model = rb_property_view_get_model (source->priv->artists);
-	RhythmDBPropertyModel *album_model = rb_property_view_get_model (source->priv->albums);
-
-	rb_debug ("query complete; resetting data models");
-
-	if (source->priv->query_type >= RB_LIBRARY_QUERY_TYPE_GENRE)
-		g_object_set (G_OBJECT (genre_model), "query-model", model, NULL);
-	if (source->priv->query_type >= RB_LIBRARY_QUERY_TYPE_ARTIST)
-		g_object_set (G_OBJECT (artist_model), "query-model", model, NULL);
-	if (source->priv->query_type >= RB_LIBRARY_QUERY_TYPE_ALBUM)
-		g_object_set (G_OBJECT (album_model), "query-model", model, NULL);
-	if (genre_model != source->priv->cached_genres_model)
-		g_object_set (G_OBJECT (source->priv->cached_genres_model),
-			      "query-model", model, NULL);
-	if (artist_model != source->priv->cached_artists_model)
-		g_object_set (G_OBJECT (source->priv->cached_artists_model),
-			      "query-model", model, NULL);
-	if (album_model != source->priv->cached_albums_model)
-		g_object_set (G_OBJECT (source->priv->cached_albums_model),
-			      "query-model", model, NULL);
-	source->priv->active_query = NULL;
-}
-
-/* static void */
-/* rb_library_source_drop_cb (GtkWidget *widget, */
-/* 			 GdkDragContext *context, */
-/* 			 gint x, */
-/* 			 gint y, */
-/* 			 GtkSelectionData *data, */
-/* 			 guint info, */
-/* 			 guint time, */
-/* 			 gpointer user_data) */
-/* { */
-/* 	RBLibrarySource *source = RB_LIBRARY_SOURCE (user_data); */
-/* 	GtkTargetList *tlist; */
-/* 	gboolean ret; */
-
-/* 	rb_debug ("checking target"); */
-
-/* 	tlist = gtk_target_list_new (target_uri, 1); */
-/* 	ret = (gtk_drag_dest_find_target (widget, context, tlist) != GDK_NONE); */
-/* 	gtk_target_list_unref (tlist); */
-
-/* 	if (ret == FALSE) */
-/* 		return; */
-
-/* 	if (!impl_receive_drag (RB_SOURCE (source), data)) { */
-/* 		gtk_drag_finish (context, FALSE, FALSE, time); */
-/* 		return; */
-/* 	} */
-
-/* 	gtk_drag_finish (context, TRUE, FALSE, time); */
-/* } */
