@@ -23,10 +23,10 @@
 #define __RB_LIBRARY_SOURCE_H
 
 #include <gtk/gtkwindow.h>
-#include <bonobo/bonobo-ui-container.h>
+#include <bonobo/bonobo-ui-component.h>
 
 #include "rb-source.h"
-#include "rb-library.h"
+#include "rhythmdb.h"
 
 G_BEGIN_DECLS
 
@@ -51,15 +51,13 @@ typedef struct
 	RBSourceClass parent;
 } RBLibrarySourceClass;
 
-GType		rb_library_source_get_type (void);
+GType		rb_library_source_get_type	(void);
 
-RBSource *	rb_library_source_new      (RBLibrary *library);
-void		rb_library_source_show_browser (RBLibrarySource *source,
-						gboolean show);
+RBSource *	rb_library_source_new		(RhythmDB *db, BonoboUIComponent *component);
+void		rb_library_source_show_browser	(RBLibrarySource *source,
+						 gboolean show);
 
-void		rb_library_source_add_location (RBLibrarySource *source, GtkWindow *win);
-
-gboolean	rb_library_source_eval_filter (RBLibrarySource *source, RBNode *node);
+void		rb_library_source_add_location	(RBLibrarySource *source, GtkWindow *win);
 
 G_END_DECLS
 
