@@ -91,6 +91,7 @@ static void rb_playlist_source_add_list_uri (RBPlaylistSource *source,
 
 #define PLAYLIST_SOURCE_SONGS_POPUP_PATH "/popups/PlaylistSongsList"
 #define PLAYLIST_SOURCE_POPUP_PATH "/popups/PlaylistSourceList"
+#define PLAYLIST_SOURCE_AUTOMATIC_POPUP_PATH "/popups/AutomaticPlaylistSourceList"
 
 struct RBPlaylistSourcePrivate
 {
@@ -447,6 +448,20 @@ rb_playlist_source_set_query (RBPlaylistSource *source,
 	rb_entry_view_poll_model (source->priv->songs);
 }
 
+void
+rb_playlist_source_get_query (RBPlaylistSource *source,
+			      GPtrArray **query,
+			      guint *limit_count,
+			      guint *limit_mb)
+{
+	g_assert (source->priv->automatic);
+
+	g_object_get (G_OBJECT (source->priv->model),
+		      "query", query,
+		      "max-count", limit_count,
+		      "max-size", limit_mb, NULL);
+}
+
 static const char *
 impl_get_status (RBSource *asource)
 {
@@ -536,25 +551,66 @@ impl_song_properties (RBSource *asource)
 		rb_debug ("failed to create dialog, or no selection!");
 }
 
+static RhythmDBPropType
+rb_playlist_source_drag_atom_to_prop (GdkAtom smasher)
+{
+	if (smasher == gdk_atom_intern ("text/x-rhythmbox-album", TRUE))
+		return RHYTHMDB_PROP_ALBUM;
+	else if (smasher == gdk_atom_intern ("text/x-rhythmbox-artist", TRUE))
+		return RHYTHMDB_PROP_ARTIST;
+	else if (smasher == gdk_atom_intern ("text/x-rhythmbox-genre", TRUE))
+		return RHYTHMDB_PROP_GENRE;
+	else {
+		g_assert_not_reached ();
+		return 0;
+	}
+}
+
 static gboolean
 impl_receive_drag (RBSource *asource, GtkSelectionData *data)
 {
 	GList *list;
 	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (asource);
-	list = gnome_vfs_uri_list_parse (data->data);
 
-	if (list != NULL)
-		rb_playlist_source_add_list_uri (source, list);
-	else
-		return FALSE;
 
-	return TRUE;
+        if (data->type == gdk_atom_intern ("text/uri-list", TRUE)) {
+                list = gnome_vfs_uri_list_parse (data->data);
+
+                if (list != NULL)
+                        rb_playlist_source_add_list_uri (source, list);
+                else
+                        return FALSE;
+
+	} else {
+		GPtrArray *query;
+		GPtrArray *subquery;
+
+		subquery = rhythmdb_query_parse (source->priv->db,
+						 RHYTHMDB_QUERY_PROP_EQUALS,
+						 rb_playlist_source_drag_atom_to_prop (data->type),
+						 data->data,
+						 RHYTHMDB_QUERY_END);
+		query = rhythmdb_query_parse (source->priv->db,
+					      RHYTHMDB_QUERY_PROP_EQUALS,
+					      RHYTHMDB_PROP_TYPE,
+					      RHYTHMDB_ENTRY_TYPE_SONG,
+					      RHYTHMDB_QUERY_SUBQUERY,
+					      subquery,
+					      RHYTHMDB_QUERY_END);
+                rb_playlist_source_set_query (source, query, 0, 0);
+	}
+
+        return TRUE;
 }
 
 static gboolean
-impl_show_popup (RBSource *source)
+impl_show_popup (RBSource *asource)
 {
-	rb_bonobo_show_popup (GTK_WIDGET (source), PLAYLIST_SOURCE_POPUP_PATH);
+	RBPlaylistSource *source = RB_PLAYLIST_SOURCE (asource);
+	if (source->priv->automatic)
+		rb_bonobo_show_popup (GTK_WIDGET (source), PLAYLIST_SOURCE_AUTOMATIC_POPUP_PATH);
+	else
+		rb_bonobo_show_popup (GTK_WIDGET (source), PLAYLIST_SOURCE_POPUP_PATH);
 	return TRUE;
 }
 
