@@ -31,8 +31,7 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <string.h>
 #include <stdlib.h>
-
-#include "monkey-media-audio-quality.h"
+#include <math.h>
 
 #include "rb-tree-dnd.h"
 #include "rb-tree-view-column.h"
@@ -40,7 +39,6 @@
 #include "rb-dialog.h"
 #include "rb-debug.h"
 #include "rhythmdb.h"
-#include "rhythmdb-model.h"
 #include "rhythmdb-query-model.h"
 #include "rb-cell-renderer-pixbuf.h"
 #include "rb-cell-renderer-rating.h"
@@ -105,7 +103,7 @@ static void rb_entry_view_sort_key_changed_cb (GConfClient* client,
 /* 						  gpointer user_data); */
 static void rb_entry_view_rated_cb (RBCellRendererRating *cellrating,
 				   const char *path,
-				   int rating,
+				   double rating,
 				   RBEntryView *view);
 static gboolean rb_entry_view_button_press_cb (GtkTreeView *treeview,
 					      GdkEventButton *event,
@@ -114,7 +112,6 @@ static gboolean rb_entry_view_entry_is_visible (RBEntryView *view, RhythmDBEntry
 						GtkTreeIter *iter);
 static void rb_entry_view_scroll_to_iter (RBEntryView *view,
 					  GtkTreeIter *iter);
-static gboolean idle_poll_model (RBEntryView *view);
 
 struct RBEntryViewReverseSortingData
 {
@@ -128,13 +125,13 @@ struct RBEntryViewPrivate
 {
 	RhythmDB *db;
 	
-	RhythmDBModel *model;
+	RhythmDBQueryModel *model;
 
 	GtkWidget *treeview;
 	GtkTreeSelection *selection;
 
 	gboolean playing;
-	RhythmDBModel *playing_model;
+	RhythmDBQueryModel *playing_model;
 	RhythmDBEntry *playing_entry;
 	gboolean playing_entry_in_view;
 	GtkTreeIter playing_entry_iter;
@@ -151,6 +148,7 @@ struct RBEntryViewPrivate
 	GtkTreeViewColumn *sorting_column;
 	gint sorting_order;
 	struct RBEntryViewReverseSortingData *reverse_sorting_data;
+	gboolean resorting;
 
 	gboolean have_selection;
 
@@ -171,8 +169,6 @@ struct RBEntryViewPrivate
 
 	gboolean idle;
 
-	guint model_poll_id;
-
 #ifdef USE_GTK_TREE_VIEW_WORKAROUND	
 	guint freeze_count;
 #endif
@@ -182,6 +178,7 @@ enum
 {
 	ENTRY_ADDED,
 	ENTRY_DELETED,
+	ENTRIES_REPLACED,
 	ENTRY_SELECTED,
 	ENTRY_ACTIVATED,
 	CHANGED,
@@ -258,9 +255,9 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_MODEL,
 					 g_param_spec_object ("model",
-							      "RhythmDBModel",
-							      "RhythmDBModel",
-							      RHYTHMDB_TYPE_MODEL,
+							      "RhythmDBQueryModel",
+							      "RhythmDBQueryModel",
+							      RHYTHMDB_TYPE_QUERY_MODEL,
 							      G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
@@ -310,6 +307,15 @@ rb_entry_view_class_init (RBEntryViewClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_POINTER);
+	rb_entry_view_signals[ENTRIES_REPLACED] =
+		g_signal_new ("entries-replaced",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBEntryViewClass, entries_replaced),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
 	rb_entry_view_signals[ENTRY_ACTIVATED] =
 		g_signal_new ("entry-activated",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -417,8 +423,6 @@ rb_entry_view_finalize (GObject *object)
 	if (view->priv->change_sig_queued)
 		g_source_remove (view->priv->change_sig_id);
 
-	g_source_remove (view->priv->model_poll_id);
-
 	if (view->priv->gconf_notification_id > 0)
 		eel_gconf_notification_remove (view->priv->gconf_notification_id);
 	if (view->priv->sorting_gconf_notification_id > 0)
@@ -533,12 +537,21 @@ rb_entry_view_set_property (GObject *object,
 		break;
 	case PROP_MODEL:
 	{
-		RhythmDBModel *new_model;
+		RhythmDBQueryModel *new_model;
 		struct RBEntryViewColumnSortData *sort_data;
 		
 		if (view->priv->model) {
-			rhythmdb_model_cancel (view->priv->model);
+			rhythmdb_query_model_cancel (view->priv->model);
 			rhythmdb_query_model_set_connected (RHYTHMDB_QUERY_MODEL (view->priv->model), FALSE);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->model),
+							      G_CALLBACK (rb_entry_view_row_inserted_cb),
+							      view);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->model),
+							      G_CALLBACK (rb_entry_view_row_deleted_cb),
+							      view);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (view->priv->model),
+							      G_CALLBACK (rb_entry_view_row_changed_cb),
+							      view);
 		}
 		new_model = g_value_get_object (value);
 
@@ -586,6 +599,15 @@ rb_entry_view_set_property (GObject *object,
 					 GTK_TREE_MODEL (new_model));
 		view->priv->model = new_model;
 		view->priv->have_selection = FALSE;
+
+		if (view->priv->resorting) {
+			/* When the sort order changes, the model is replaced
+			 * but the set of entries doesn't change. */
+			view->priv->resorting = FALSE;
+		} else {
+			g_signal_emit (G_OBJECT (view), rb_entry_view_signals[ENTRIES_REPLACED], 0);
+		}
+
 		queue_changed_sig (view);
 
 		break;
@@ -615,9 +637,9 @@ rb_entry_view_set_property (GObject *object,
 
 		if (view->priv->playing_entry != NULL) {
 			view->priv->playing_entry_in_view = 
-				rhythmdb_model_entry_to_iter (view->priv->model,
-							  view->priv->playing_entry,
-							  &view->priv->playing_entry_iter);
+				rhythmdb_query_model_entry_to_iter (view->priv->model,
+								    view->priv->playing_entry,
+								    &view->priv->playing_entry_iter);
 			if (view->priv->playing_entry_in_view) {
 				path = gtk_tree_model_get_path (GTK_TREE_MODEL (view->priv->model),
 								&view->priv->playing_entry_iter);
@@ -702,7 +724,7 @@ rb_entry_view_new (RhythmDB *db, const char *sort_key,
 }
 
 void
-rb_entry_view_set_model (RBEntryView *view, RhythmDBModel *model)
+rb_entry_view_set_model (RBEntryView *view, RhythmDBQueryModel *model)
 {
 	g_object_set (G_OBJECT (view), "model", model, NULL);
 }
@@ -711,7 +733,7 @@ gboolean
 rb_entry_view_busy (RBEntryView *view)
 {
 	return view->priv->model &&
-		rhythmdb_model_has_pending_changes (RHYTHMDB_MODEL (view->priv->model));
+		rhythmdb_query_model_has_pending_changes (RHYTHMDB_QUERY_MODEL (view->priv->model));
 }
 
 glong
@@ -732,7 +754,7 @@ entry_from_tree_path (RBEntryView *view, GtkTreePath *path)
 	GtkTreeIter entry_iter;
 	RhythmDBEntry *entry;
 
-	gtk_tree_model_get_iter (GTK_TREE_MODEL (view->priv->model), &entry_iter, path);
+	g_assert (gtk_tree_model_get_iter (GTK_TREE_MODEL (view->priv->model), &entry_iter, path));
 	gtk_tree_model_get (GTK_TREE_MODEL (view->priv->model), &entry_iter, 0,
 			    &entry, -1);
 	return entry;
@@ -747,13 +769,6 @@ entry_from_tree_iter (RBEntryView *view, GtkTreeIter *iter)
 	return entry;
 }
 
-RhythmDBModel *
-rb_entry_view_get_model (RBEntryView *view)
-{
-	g_return_val_if_fail (RB_IS_ENTRY_VIEW (view), NULL);
-	return view->priv->model;
-}
-
 static gint
 reverse_sorting_func (gpointer a, gpointer b, struct RBEntryViewReverseSortingData *data)
 {
@@ -766,50 +781,6 @@ struct RBEntryViewCellDataFuncData {
 	RhythmDBPropType propid;
 };
 
-static gint
-rb_entry_view_artist_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
-				RBEntryView *view)
-{
-	gint a_int, b_int;
-	const char *a_str = NULL;
-	const char *b_str = NULL;
-	gint ret;
-
-	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_ARTIST_SORT_KEY);
-	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_ARTIST_SORT_KEY);
-
-	ret = strcmp (a_str, b_str);
-	if (ret != 0)
-		goto out;
-
-	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_ALBUM_SORT_KEY);
-	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_ALBUM_SORT_KEY);
-
-	ret = strcmp (a_str, b_str);
-	if (ret != 0)
-		goto out;
-
-	a_int = rhythmdb_entry_get_int (view->priv->db, a, RHYTHMDB_PROP_TRACK_NUMBER);
-	b_int = rhythmdb_entry_get_int (view->priv->db, b, RHYTHMDB_PROP_TRACK_NUMBER);
-
-	if (a_int != b_int) {
-		ret = (a_int < b_int ? -1 : 1);
-		goto out;
-	}
-
-	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_TITLE_SORT_KEY);
-	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_TITLE_SORT_KEY);
-
-	ret = strcmp (a_str, b_str);
-	if (ret != 0)
-		goto out;
-
-	ret = 0;
-
-out:
-	return ret;
-}
-
 static gint
 rb_entry_view_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 			       RBEntryView *view)
@@ -824,18 +795,54 @@ rb_entry_view_album_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 
 	ret = strcmp (a_str, b_str);
 	if (ret != 0)
-		goto out;
+		return ret;
 
 	a_int = rhythmdb_entry_get_int (view->priv->db, a, RHYTHMDB_PROP_TRACK_NUMBER);
 	b_int = rhythmdb_entry_get_int (view->priv->db, b, RHYTHMDB_PROP_TRACK_NUMBER);
 
-	if (a_int != b_int) {
-		ret = (a_int < b_int ? -1 : 1);
-		goto out;
-	}
+	if (a_int != b_int)
+		return (a_int < b_int ? -1 : 1);
 
-out:
-	return ret;
+	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_TITLE_SORT_KEY);
+	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_TITLE_SORT_KEY);
+
+	return strcmp (a_str, b_str);
+}
+
+static gint
+rb_entry_view_artist_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+				RBEntryView *view)
+{
+	const char *a_str = NULL;
+	const char *b_str = NULL;
+	gint ret;
+
+	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_ARTIST_SORT_KEY);
+	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_ARTIST_SORT_KEY);
+
+	ret = strcmp (a_str, b_str);
+	if (ret != 0)
+		return ret;
+
+	return rb_entry_view_album_sort_func (a, b, view);
+}
+
+static gint
+rb_entry_view_genre_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+			       RBEntryView *view)
+{
+	const char *a_str = NULL;
+	const char *b_str = NULL;
+	gint ret;
+
+	a_str = rhythmdb_entry_get_string (view->priv->db, a, RHYTHMDB_PROP_GENRE_SORT_KEY);
+	b_str = rhythmdb_entry_get_string (view->priv->db, b, RHYTHMDB_PROP_GENRE_SORT_KEY);
+
+	ret = strcmp (a_str, b_str);
+	if (ret != 0)
+		return ret;
+
+	return rb_entry_view_artist_sort_func (a, b, view);
 }
 
 static gint
@@ -882,6 +889,21 @@ rb_entry_view_int_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 	return ret;
 }
 
+static gint
+rb_entry_view_double_ceiling_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
+				       struct RBEntryViewCellDataFuncData *data)
+{
+	gdouble a_val, b_val;
+	gint ret;
+
+	a_val = ceil (rhythmdb_entry_get_double (data->view->priv->db, a, data->propid));
+	b_val = ceil (rhythmdb_entry_get_double (data->view->priv->db, b, data->propid));
+
+	ret = (a_val == b_val ? 0 : (a_val > b_val ? 1 : -1));
+
+	return ret;
+}
+
 static gint
 rb_entry_view_long_sort_func (RhythmDBEntry *a, RhythmDBEntry *b,
 			      struct RBEntryViewCellDataFuncData *data)
@@ -939,13 +961,13 @@ rb_entry_view_rating_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer
 				     RBEntryView *view)
 {
 	RhythmDBEntry *entry;
-	guint rating;
+	gdouble rating;
 
 	entry = entry_from_tree_iter (view, iter);
 
 	rhythmdb_read_lock (view->priv->db);
 
-	rating = rhythmdb_entry_get_int (view->priv->db, entry, RHYTHMDB_PROP_RATING);
+	rating = rhythmdb_entry_get_double (view->priv->db, entry, RHYTHMDB_PROP_RATING);
 
 	rhythmdb_read_unlock (view->priv->db);
 
@@ -1025,7 +1047,10 @@ rb_entry_view_duration_cell_data_func (GtkTreeViewColumn *column, GtkCellRendere
 	minutes = duration / 60;
 	seconds = duration % 60;
 
-	str = g_strdup_printf (_("%d:%02d"), minutes, seconds);
+	if (minutes == 0 && seconds == 0)
+		str = g_strdup (_("Unknown"));
+	else
+		str = g_strdup_printf (_("%d:%02d"), minutes, seconds);
 
 	g_object_set (G_OBJECT (renderer), "text", str, NULL);
 
@@ -1038,24 +1063,31 @@ rb_entry_view_quality_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer
 				      struct RBEntryViewCellDataFuncData *data)
 {
 	RhythmDBEntry *entry;
-	char *str;
-	MonkeyMediaAudioQuality quality;
+	guint bitrate;
 
 	entry = entry_from_tree_iter (data->view, iter);
 
 	rhythmdb_read_lock (data->view->priv->db);
 
-	quality = rhythmdb_entry_get_int (data->view->priv->db, entry,
+	bitrate = rhythmdb_entry_get_int (data->view->priv->db, entry,
 					  data->propid);
 
 	rhythmdb_read_unlock (data->view->priv->db);
 
-	if (quality > 0) {
-		str = monkey_media_audio_quality_to_string (quality);
-		g_object_set (G_OBJECT (renderer), "text", str, NULL);
-		g_free (str);
-	} else {
+	if (bitrate <= 0) {
 		g_object_set (G_OBJECT (renderer), "text", _("Unknown"), NULL);
+	} else if (bitrate <= 80) {
+		g_object_set (G_OBJECT (renderer), "text", _("Very Low"), NULL);
+	} else if (bitrate <= 112) {
+		g_object_set (G_OBJECT (renderer), "text", _("Low"), NULL);
+	} else if (bitrate <= 160) {
+		g_object_set (G_OBJECT (renderer), "text", _("Regular"), NULL);
+	} else if (bitrate <= 224) {
+		g_object_set (G_OBJECT (renderer), "text", _("High"), NULL);
+	} else if (bitrate <= 1410) {
+		g_object_set (G_OBJECT (renderer), "text", _("Very High"), NULL);
+	} else {
+		g_object_set (G_OBJECT (renderer), "text", _("Perfect"), NULL);
 	}
 }
 
@@ -1074,9 +1106,9 @@ rb_entry_view_string_cell_data_func (GtkTreeViewColumn *column, GtkCellRenderer
 
 	str = rhythmdb_entry_get_string (data->view->priv->db, entry, data->propid);
 
-	rhythmdb_read_unlock (data->view->priv->db);
-
 	g_object_set (G_OBJECT (renderer), "text", str, NULL);
+
+	rhythmdb_read_unlock (data->view->priv->db);
 }
 
 static void
@@ -1143,6 +1175,14 @@ rb_entry_view_get_sorting_type (RBEntryView *view)
 	return eel_gconf_get_string (view->priv->sorting_key);
 }
 
+void
+rb_entry_view_set_resorting (RBEntryView *view)
+{
+	if (view->priv->resorting)
+		g_warning ("Sort order changed while resorting");
+	view->priv->resorting = TRUE;
+}
+
 static void
 rb_entry_view_column_clicked_cb (GtkTreeViewColumn *column, RBEntryView *view)
 {
@@ -1207,7 +1247,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		sort_data = g_new0 (struct RBEntryViewCellDataFuncData, 1);
 		sort_data->view = view;
 		sort_data->propid = propid;
-		sort_func = (GCompareDataFunc) rb_entry_view_int_sort_func;
+		sort_func = (GCompareDataFunc) rb_entry_view_double_ceiling_sort_func;
 
 		renderer = rb_cell_renderer_rating_new ();
 		gtk_tree_view_column_pack_start (column, renderer, TRUE);
@@ -1284,7 +1324,8 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		cell_data->propid = propid;
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_string_cell_data_func;				
 		sort_data->propid = RHYTHMDB_PROP_GENRE_SORT_KEY;
-		sort_func = (GCompareDataFunc) rb_entry_view_string_sort_func;
+		sort_func = (GCompareDataFunc) rb_entry_view_genre_sort_func;
+		real_sort_data = view;
 		title = _("Ge_nre");
 		key = "Genre";
 		rb_tree_view_column_set_expand (RB_TREE_VIEW_COLUMN (column), TRUE);
@@ -1299,7 +1340,7 @@ rb_entry_view_append_column (RBEntryView *view, RBEntryViewColumn coltype)
 		key = "Time";
 		break;
 	case RB_ENTRY_VIEW_COL_QUALITY:
-		propid = RHYTHMDB_PROP_QUALITY;
+		propid = RHYTHMDB_PROP_BITRATE;
 		cell_data->propid = propid;
 		cell_data_func = (GtkTreeCellDataFunc) rb_entry_view_quality_cell_data_func;
 		sort_data->propid = cell_data->propid;
@@ -1483,12 +1524,10 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 	{
 		RhythmDBQueryModel *query_model;
 		query_model = rhythmdb_query_model_new_empty (view->priv->db);
-		rb_entry_view_set_model (view, RHYTHMDB_MODEL (query_model));
+		rb_entry_view_set_model (view, RHYTHMDB_QUERY_MODEL (query_model));
 		g_object_unref (G_OBJECT (query_model));
 	}
 		
-	view->priv->model_poll_id = g_idle_add ((GSourceFunc) idle_poll_model, view);
-
 	return G_OBJECT (view);
 }
 
@@ -1508,14 +1547,14 @@ rb_entry_view_constructor (GType type, guint n_construct_properties,
 static void
 rb_entry_view_rated_cb (RBCellRendererRating *cellrating,
 		       const char *path_string,
-		       int rating,
+		       double rating,
 		       RBEntryView *view)
 {
 	GtkTreePath *path;
 	RhythmDBEntry *entry;
 	GValue value = { 0, };
 
-	g_return_if_fail (rating >= 1 && rating <= 5 );
+	g_return_if_fail (rating >= 0 && rating <= 5 );
 	g_return_if_fail (path_string != NULL);
 
 	path = gtk_tree_path_new_from_string (path_string);
@@ -1524,8 +1563,8 @@ rb_entry_view_rated_cb (RBCellRendererRating *cellrating,
 
 	rb_entry_view_freeze (view);
 
-	g_value_init (&value, G_TYPE_INT);
-	g_value_set_int (&value, rating);
+	g_value_init (&value, G_TYPE_DOUBLE);
+	g_value_set_double (&value, rating);
 	rhythmdb_entry_queue_set (view->priv->db, entry, RHYTHMDB_PROP_RATING,
 				  &value);
 	g_value_unset (&value);
@@ -1568,8 +1607,8 @@ rb_entry_view_get_next_from_entry (RBEntryView *view, RhythmDBEntry *entry)
 
 	g_return_val_if_fail (entry != NULL, NULL);
 
-	if (!rhythmdb_model_entry_to_iter (view->priv->model,
-					   entry, &iter)) {
+	if (!rhythmdb_query_model_entry_to_iter (view->priv->model,
+						 entry, &iter)) {
 		/* If the entry isn't in the entryview, the "next" entry is the first. */
 		return rb_entry_view_get_first_entry (view);
 	}
@@ -1589,8 +1628,8 @@ rb_entry_view_get_previous_from_entry (RBEntryView *view, RhythmDBEntry *entry)
 
 	g_return_val_if_fail (entry != NULL, NULL);
 
-	if (!rhythmdb_model_entry_to_iter (view->priv->model,
-					   entry, &iter))
+	if (!rhythmdb_query_model_entry_to_iter (view->priv->model,
+						 entry, &iter))
 		return NULL;
 
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (view->priv->model), &iter);
@@ -1692,11 +1731,16 @@ rb_entry_view_button_press_cb (GtkTreeView *treeview,
 
 		gtk_tree_view_get_path_at_pos (treeview, event->x, event->y, &path, NULL, NULL, NULL);
 		if (path != NULL) {
+			GList *selected;
 			entry = entry_from_tree_path (view, path);
+
+			selected = rb_entry_view_get_selected_entries (view);
+			
+			if (!g_list_find (selected, entry))
+				rb_entry_view_select_entry (view, entry);
+
+			g_list_free (selected);
 			
-			/* FIXME - don't require a popup to select an entry */
-			rb_entry_view_select_entry (view, entry);
-		
 			g_signal_emit (G_OBJECT (view), rb_entry_view_signals[SHOW_POPUP], 0);
 		}
 		return view->priv->have_selection;
@@ -1774,6 +1818,10 @@ rb_entry_view_row_inserted_cb (GtkTreeModel *model,
 			       GtkTreeIter *iter,
 			       RBEntryView *view)
 {
+	RhythmDBEntry *entry = entry_from_tree_path (view, path);
+
+	rb_debug ("row added");
+	g_signal_emit (G_OBJECT (view), rb_entry_view_signals[ENTRY_ADDED], 0, entry);
 	queue_changed_sig (view);
 }
 
@@ -1846,8 +1894,8 @@ rb_entry_view_select_entry (RBEntryView *view,
 
 	rb_entry_view_select_none (view);
 
-	if (rhythmdb_model_entry_to_iter (view->priv->model,
-					  entry, &iter)) {
+	if (rhythmdb_query_model_entry_to_iter (view->priv->model,
+						entry, &iter)) {
 		gtk_tree_selection_select_iter (view->priv->selection, &iter);
 	}
 
@@ -1860,8 +1908,8 @@ rb_entry_view_scroll_to_entry (RBEntryView *view,
 {
 	GtkTreeIter iter;
 	
-	if (rhythmdb_model_entry_to_iter (view->priv->model,
-					  entry, &iter)) {
+	if (rhythmdb_query_model_entry_to_iter (view->priv->model,
+						entry, &iter)) {
 		rb_entry_view_scroll_to_iter (view, &iter);
 	}
 }
@@ -1900,8 +1948,8 @@ rb_entry_view_get_entry_contained (RBEntryView *view,
 {
 	GtkTreeIter unused;
 
-	return rhythmdb_model_entry_to_iter (view->priv->model,
-					     entry, &unused);
+	return rhythmdb_query_model_entry_to_iter (view->priv->model,
+						   entry, &unused);
 }
 
 static gboolean
@@ -1917,8 +1965,8 @@ rb_entry_view_entry_is_visible (RBEntryView *view,
 	if (!GTK_WIDGET_REALIZED (view))
 		return FALSE;
 
-	if (!rhythmdb_model_entry_to_iter (view->priv->model,
-					   entry, iter))
+	if (!rhythmdb_query_model_entry_to_iter (view->priv->model,
+						 entry, iter))
 		return FALSE;
 
 	path = gtk_tree_model_get_path (GTK_TREE_MODEL (view->priv->model), iter);
@@ -2038,8 +2086,8 @@ rb_entry_view_sync_columns_visible (RBEntryView *view)
 		for (i = 0; items[i] != NULL && *(items[i]); i++) {
 			int value = propid_from_name (items[i]);
 
-			g_assert ((value >= 0) && (value < RHYTHMDB_NUM_PROPERTIES));
-			visible_properties = g_list_append (visible_properties, GINT_TO_POINTER (value));
+			if ((value >= 0) && (value < RHYTHMDB_NUM_PROPERTIES))
+				visible_properties = g_list_append (visible_properties, GINT_TO_POINTER (value));
 		}
 		g_strfreev (items);
 	}
@@ -2072,41 +2120,10 @@ gboolean
 rb_entry_view_poll_model (RBEntryView *view)
 {
 	GTimeVal timeout;
-	gboolean did_sync;
 
 	g_get_current_time (&timeout);
 	g_time_val_add (&timeout, G_USEC_PER_SEC*0.75);
 
-	did_sync = rhythmdb_model_poll (view->priv->model, &timeout);
-	if (did_sync) {
-		g_source_remove (view->priv->model_poll_id);
-		view->priv->model_poll_id =
-			g_idle_add ((GSourceFunc) idle_poll_model, view);
-	}
-	return did_sync;
+	return rhythmdb_query_model_poll (view->priv->model, &timeout);
 }
 
-static gboolean
-idle_poll_model (RBEntryView *view)
-{
-	gboolean did_sync;
-	GTimeVal timeout;
-
-	g_get_current_time (&timeout);
-	g_time_val_add (&timeout, G_USEC_PER_SEC*0.75);
-
-	GDK_THREADS_ENTER ();
-
-	did_sync = rhythmdb_model_poll (view->priv->model, &timeout);
-
-	if (did_sync)
-		view->priv->model_poll_id =
-			g_idle_add_full (G_PRIORITY_LOW, (GSourceFunc) idle_poll_model, view, NULL);
-	else
-		view->priv->model_poll_id =
-			g_timeout_add (300, (GSourceFunc) idle_poll_model, view);
-
-	GDK_THREADS_LEAVE ();
-
-	return FALSE;
-}
