@@ -36,25 +36,36 @@
 typedef struct
 {
 	const char *name;
-	int val;
-} RBQueryCreatorOption;
+	RhythmDBPropType strict_val;
+	RhythmDBPropType fuzzy_val;
+} RBQueryCreatorPropertyOption;
 
-const RBQueryCreatorOption property_options[] =
+typedef struct
 {
-	{ N_("Title"),	RHYTHMDB_PROP_TITLE_FOLDED },
-	{ N_("Artist"),	RHYTHMDB_PROP_ARTIST_FOLDED },
-	{ N_("Album"),	RHYTHMDB_PROP_ALBUM_FOLDED },
-	{ N_("Genre"),	RHYTHMDB_PROP_GENRE_FOLDED },
+	const char *name;
+	gboolean strict;
+	RhythmDBPropType val;
+} RBQueryCreatorCriteriaOption;
+
+const RBQueryCreatorPropertyOption property_options[] =
+{
+	{ N_("Title"), RHYTHMDB_PROP_TITLE, RHYTHMDB_PROP_TITLE_FOLDED },
+	{ N_("Artist"), RHYTHMDB_PROP_ARTIST, RHYTHMDB_PROP_ARTIST_FOLDED },
+	{ N_("Album"), RHYTHMDB_PROP_ALBUM, RHYTHMDB_PROP_ALBUM_FOLDED },
+	{ N_("Genre"), RHYTHMDB_PROP_GENRE, RHYTHMDB_PROP_GENRE_FOLDED },
 };
 
-const RBQueryCreatorOption criteria_options[] =
+const RBQueryCreatorCriteriaOption criteria_options[] =
 {
-	{ N_("contains"),		RHYTHMDB_QUERY_PROP_LIKE },
-	{ N_("does not contain"),	RHYTHMDB_QUERY_PROP_NOT_LIKE },
+	{ N_("contains"), 0, RHYTHMDB_QUERY_PROP_LIKE },
+	{ N_("does not contain"), 0, RHYTHMDB_QUERY_PROP_NOT_LIKE },
+	{ N_("equals"), 1, RHYTHMDB_QUERY_PROP_EQUALS },
 };
 
 static void rb_query_creator_class_init (RBQueryCreatorClass *klass);
 static void rb_query_creator_init (RBQueryCreator *dlg);
+static GObject *rb_query_creator_constructor (GType type, guint n_construct_properties,
+					      GObjectConstructParam *construct_properties);
 static void rb_query_creator_finalize (GObject *object);
 static void rb_query_creator_set_property (GObject *object,
 					      guint prop_id,
@@ -64,12 +75,26 @@ static void rb_query_creator_get_property (GObject *object,
 					      guint prop_id,
 					      GValue *value,
 					      GParamSpec *pspec);
-static void setup_option_menu (GtkWidget *option_menu,
-			       const RBQueryCreatorOption *options,
-			       int length, gboolean activate_first);
-static GtkWidget * create_option_menu (const RBQueryCreatorOption *options,
-				      int length, gboolean activate_first);
+static void select_property_from_value (GtkWidget *option_menu,
+					const RBQueryCreatorPropertyOption *options,
+					int length, RhythmDBPropType prop);
+static void select_criteria_from_value (GtkWidget *option_menu,
+					const RBQueryCreatorCriteriaOption *options,
+					int length, RhythmDBQueryType qtype);
+
+static void setup_property_option_menu (GtkWidget *option_menu,
+					const RBQueryCreatorPropertyOption *options,
+					int length, gboolean activate_first);
+static void setup_criteria_option_menu (GtkWidget *option_menu,
+					const RBQueryCreatorCriteriaOption *options,
+					int length, gboolean activate_first);
+static GtkWidget * create_property_option_menu (const RBQueryCreatorPropertyOption *options,
+						int length, gboolean activate_first);
+static GtkWidget * create_criteria_option_menu (const RBQueryCreatorCriteriaOption *options,
+						int length, gboolean activate_first);
 static GtkWidget * option_menu_get_active_child (GtkWidget *option_menu);
+static GtkWidget * get_box_widget_at_pos (GtkBox *box, guint pos);
+
 static void append_row (RBQueryCreator *dialog);
 static void add_button_click_cb (GtkWidget *button, RBQueryCreator *creator);
 static void remove_button_click_cb (GtkWidget *button, RBQueryCreator *creator);
@@ -78,6 +103,8 @@ static void limit_toggled_cb (GtkWidget *limit, RBQueryCreator *creator);
 struct RBQueryCreatorPrivate
 {
 	RhythmDB *db;
+
+	gboolean creating;
 	
 	GtkSizeGroup *property_size_group;
 	GtkSizeGroup *criteria_size_group;
@@ -100,6 +127,7 @@ enum
 {
 	PROP_0,
 	PROP_DB,
+	PROP_CREATING,
 };
 
 GType
@@ -138,6 +166,7 @@ rb_query_creator_class_init (RBQueryCreatorClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = rb_query_creator_finalize;
+	object_class->constructor = rb_query_creator_constructor;
 	object_class->set_property = rb_query_creator_set_property;
 	object_class->get_property = rb_query_creator_get_property;
 
@@ -148,11 +177,23 @@ rb_query_creator_class_init (RBQueryCreatorClass *klass)
 							      "RhythmDB database",
 							      RHYTHMDB_TYPE,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property (object_class,
+					 PROP_CREATING,
+					 g_param_spec_boolean ("creating",
+							       "creating",
+							       "Whether or not we're creating a new playlist",
+							       TRUE,
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 }
 
-static void
-rb_query_creator_init (RBQueryCreator *dlg)
+static GObject *
+rb_query_creator_constructor (GType type, guint n_construct_properties,
+			      GObjectConstructParam *construct_properties)
 {
+	RBQueryCreator *dlg;
+	RBQueryCreatorClass *klass;
+	GObjectClass *parent_class;  
 	GladeXML *xml;
 	GtkWidget *mainbox;
 	GtkBox *hbox;
@@ -160,16 +201,24 @@ rb_query_creator_init (RBQueryCreator *dlg)
 	GtkWidget *first_criteria;
 	GtkWidget *first_entry;
 
-	dlg->priv = g_new0 (RBQueryCreatorPrivate, 1);
 
-	dlg->priv->queries = g_ptr_array_new ();
-
-	gtk_dialog_add_button (GTK_DIALOG (dlg),
-			       GTK_STOCK_CANCEL,
-			       GTK_RESPONSE_CLOSE);
-	gtk_dialog_add_button (GTK_DIALOG (dlg),
-			       GTK_STOCK_NEW,
-			       GTK_RESPONSE_OK);
+	klass = RB_QUERY_CREATOR_CLASS (g_type_class_peek (type));
+	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
+	dlg = RB_QUERY_CREATOR (parent_class->constructor (type, n_construct_properties,
+							   construct_properties));
+	
+	if (dlg->priv->creating) { 
+		gtk_dialog_add_button (GTK_DIALOG (dlg),
+				       GTK_STOCK_CANCEL,
+				       GTK_RESPONSE_CLOSE);
+		gtk_dialog_add_button (GTK_DIALOG (dlg),
+				       GTK_STOCK_NEW,
+				       GTK_RESPONSE_OK);
+	} else {
+		gtk_dialog_add_button (GTK_DIALOG (dlg),
+				       GTK_STOCK_CLOSE,
+				       GTK_RESPONSE_CLOSE);
+	}
 	gtk_dialog_set_default_response (GTK_DIALOG (dlg),
 					 GTK_RESPONSE_CLOSE);
 
@@ -203,13 +252,13 @@ rb_query_creator_init (RBQueryCreator *dlg)
 	gtk_size_group_add_widget (dlg->priv->button_size_group, dlg->priv->addbutton);
 	g_signal_connect (G_OBJECT (dlg->priv->addbutton), "clicked", G_CALLBACK (add_button_click_cb),
 			  dlg);
-	first_option = create_option_menu (property_options,
-					   G_N_ELEMENTS (property_options),
-					   FALSE);
+	first_option = create_property_option_menu (property_options,
+						    G_N_ELEMENTS (property_options),
+						    FALSE);
 	gtk_size_group_add_widget (dlg->priv->property_size_group, first_option);
-	first_criteria = create_option_menu (criteria_options,
-					     G_N_ELEMENTS (criteria_options),
-					     FALSE);
+	first_criteria = create_criteria_option_menu (criteria_options,
+						      G_N_ELEMENTS (criteria_options),
+						      FALSE);
 	gtk_size_group_add_widget (dlg->priv->criteria_size_group, first_criteria);
 	first_entry = gtk_entry_new ();
 	gtk_size_group_add_widget (dlg->priv->entry_size_group, first_entry);
@@ -224,6 +273,14 @@ rb_query_creator_init (RBQueryCreator *dlg)
 	mainbox = glade_xml_get_widget (xml, "main_vbox");
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), mainbox, FALSE, FALSE, 0);
 	gtk_widget_show_all (GTK_WIDGET (dlg));
+
+	return G_OBJECT (dlg);
+}
+
+static void
+rb_query_creator_init (RBQueryCreator *dlg)
+{
+	dlg->priv = g_new0 (RBQueryCreatorPrivate, 1);
 }
 
 static void
@@ -238,8 +295,6 @@ rb_query_creator_finalize (GObject *object)
 
 	g_return_if_fail (dlg->priv != NULL);
 
-	g_ptr_array_free (dlg->priv->queries, TRUE);
-
 	g_object_unref (G_OBJECT (dlg->priv->property_size_group));
 	g_object_unref (G_OBJECT (dlg->priv->criteria_size_group));
 	g_object_unref (G_OBJECT (dlg->priv->entry_size_group));
@@ -252,9 +307,9 @@ rb_query_creator_finalize (GObject *object)
 
 static void
 rb_query_creator_set_property (GObject *object,
-				  guint prop_id,
-				  const GValue *value,
-				  GParamSpec *pspec)
+			       guint prop_id,
+			       const GValue *value,
+			       GParamSpec *pspec)
 {
 	RBQueryCreator *dlg = RB_QUERY_CREATOR (object);
 
@@ -263,6 +318,9 @@ rb_query_creator_set_property (GObject *object,
 	case PROP_DB:
 		dlg->priv->db = g_value_get_object (value);
 		break;
+	case PROP_CREATING:
+		dlg->priv->creating = g_value_get_boolean (value);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -282,6 +340,9 @@ rb_query_creator_get_property (GObject *object,
 	case PROP_DB:
 		g_value_set_object (value, dlg->priv->db);
 		break;
+	case PROP_CREATING:
+		g_value_set_boolean (value, dlg->priv->creating);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -295,6 +356,69 @@ rb_query_creator_new (RhythmDB *db)
 	return g_object_new (RB_TYPE_QUERY_CREATOR, "db", db, NULL);
 }
 
+static void
+rb_query_creator_load_query (RBQueryCreator *creator, GPtrArray *query,
+			     int limit_count, int limit_size)
+{
+	int i;
+	GList *rows;
+	gboolean disjunction = FALSE;
+	RhythmDBQueryData *qdata;
+	GPtrArray *subquery;
+
+	g_assert (query->len == 2);
+
+	qdata = g_ptr_array_index (query, 1);
+	g_assert (qdata->type == RHYTHMDB_QUERY_SUBQUERY);
+
+	subquery = qdata->subquery;
+
+	for (i = 0; i < subquery->len - 1; i++) {
+		RhythmDBQueryData *data = g_ptr_array_index (subquery, i);
+		if (data->type != RHYTHMDB_QUERY_DISJUNCTION)
+			append_row (creator);
+	}
+	
+	rows = gtk_container_get_children (GTK_CONTAINER (creator->priv->vbox));
+
+	for (i = 0; i < subquery->len; i++) {
+		RhythmDBQueryData *data = g_ptr_array_index (subquery, i);
+		GtkOptionMenu *propmenu = GTK_OPTION_MENU (get_box_widget_at_pos (GTK_BOX (rows->data),
+										  0));
+		GtkOptionMenu *criteria_menu = GTK_OPTION_MENU (get_box_widget_at_pos (GTK_BOX (rows->data),
+										       1));
+		GtkEntry *text = GTK_ENTRY (get_box_widget_at_pos (GTK_BOX (rows->data), 2));
+
+		if (data->type == RHYTHMDB_QUERY_DISJUNCTION) {
+			disjunction = TRUE;
+			continue;
+		}
+		
+		select_property_from_value (GTK_WIDGET (propmenu), property_options,
+					    G_N_ELEMENTS (property_options), data->propid);
+		select_criteria_from_value (GTK_WIDGET (criteria_menu), criteria_options,
+					    G_N_ELEMENTS (criteria_options), data->type);
+		gtk_entry_set_text (text, g_value_get_string (data->val));
+		rows = rows->next;
+	}
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (creator->priv->disjunction_check),
+				      disjunction);
+}
+
+GtkWidget *
+rb_query_creator_new_from_query (RhythmDB *db, GPtrArray *query,
+				 int limit_count, int limit_size)
+{
+	RBQueryCreator *creator = g_object_new (RB_TYPE_QUERY_CREATOR, "db", db,
+						"creating", FALSE, NULL);
+	if (!creator)
+		return NULL;
+
+	rb_query_creator_load_query (creator, query, limit_count, limit_size);
+	return GTK_WIDGET (creator);
+}
+
 static GtkWidget *
 get_box_widget_at_pos (GtkBox *box, guint pos)
 {
@@ -315,12 +439,10 @@ get_box_widget_at_pos (GtkBox *box, guint pos)
 }
 
 static int
-extract_option_menu_val (GtkOptionMenu *menu)
+extract_option_menu_index (GtkOptionMenu *menu)
 {
 	GtkWidget *active_item = option_menu_get_active_child (GTK_WIDGET (menu));
-
-	return GPOINTER_TO_INT (g_object_get_data (G_OBJECT (active_item),
-						   "rb-query-creator-value"));
+	return GPOINTER_TO_INT (g_object_get_data (G_OBJECT (active_item), "rb-query-creator-value"));
 }
 
 GPtrArray *
@@ -348,10 +470,12 @@ rb_query_creator_get_query (RBQueryCreator *dlg)
 		GtkOptionMenu *criteria_menu = GTK_OPTION_MENU (get_box_widget_at_pos (GTK_BOX (row->data),
 										       1));
 		GtkEntry *text = GTK_ENTRY (get_box_widget_at_pos (GTK_BOX (row->data), 2));
-		RhythmDBPropType prop = extract_option_menu_val (propmenu);
-		RhythmDBQueryType criteria = extract_option_menu_val (criteria_menu);
+		const RBQueryCreatorPropertyOption *prop_option = &property_options[extract_option_menu_index (propmenu)];
+		const RBQueryCreatorCriteriaOption *criteria_option = &criteria_options[extract_option_menu_index (criteria_menu)];
+		RhythmDBQueryType criteria = criteria_option->val;
+		RhythmDBPropType prop = criteria_option->strict ? prop_option->strict_val : prop_option->fuzzy_val;
 		const char *data = gtk_entry_get_text (GTK_ENTRY (text));
-		char *folded_data = g_utf8_casefold (data, -1);
+		char *folded_data = criteria_option->strict ? g_strdup (data) : g_utf8_casefold (data, -1);
 
 		if (disjunction && row->next)
 			rhythmdb_query_append (dlg->priv->db,
@@ -470,15 +594,15 @@ append_row (RBQueryCreator *dialog)
 	gtk_box_reorder_child (dialog->priv->vbox, GTK_WIDGET (hbox), -1);
 
 	/* This is the main (leftmost) GtkOptionMenu, for types. */
-	option = create_option_menu (property_options,
-				     G_N_ELEMENTS (property_options),
-				     FALSE);
+	option = create_property_option_menu (property_options,
+					      G_N_ELEMENTS (property_options),
+					      FALSE);
 	gtk_size_group_add_widget (dialog->priv->property_size_group, option);
 	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (option));
 	gtk_option_menu_set_history (GTK_OPTION_MENU (option), 0);
-	criteria = create_option_menu (criteria_options,
-				       G_N_ELEMENTS (criteria_options),
-				       FALSE);
+	criteria = create_criteria_option_menu (criteria_options,
+						G_N_ELEMENTS (criteria_options),
+						FALSE);
 	gtk_size_group_add_widget (dialog->priv->criteria_size_group, criteria);
 	gtk_box_pack_start_defaults (hbox, GTK_WIDGET (criteria));
 
@@ -492,11 +616,43 @@ append_row (RBQueryCreator *dialog)
 	gtk_widget_show_all (GTK_WIDGET (dialog->priv->vbox));
 }
 
+static void
+select_property_from_value (GtkWidget *option_menu,
+			    const RBQueryCreatorPropertyOption *options,
+			    int length, RhythmDBPropType prop)
+{
+	int i;
+	
+	for (i = 0; i < length; i++) {
+		if (prop == options[i].strict_val || prop == options[i].fuzzy_val) {
+			gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), i);
+			return;
+		}
+	}
+	g_assert_not_reached ();
+}
+
+static void
+select_criteria_from_value (GtkWidget *option_menu,
+			    const RBQueryCreatorCriteriaOption *options,
+			    int length, RhythmDBQueryType qtype)
+{
+	int i;
+	
+	for (i = 0; i < length; i++) {
+		if (qtype == options[i].val) {
+			gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), i);
+			return;
+		}
+	}
+	g_assert_not_reached ();
+}
+
 /* Stolen from jamboree */
 static void
-setup_option_menu (GtkWidget *option_menu,
-		   const RBQueryCreatorOption *options,
-		   int length, gboolean activate_first)
+setup_property_option_menu (GtkWidget *option_menu,
+			    const RBQueryCreatorPropertyOption *options,
+			    int length, gboolean activate_first)
 {
 	GtkWidget *menu;
 	GtkWidget *menu_item;
@@ -508,7 +664,33 @@ setup_option_menu (GtkWidget *option_menu,
 	for (i = 0; i < length; i++) {
 		menu_item = gtk_menu_item_new_with_label (_(options[i].name));
 		g_object_set_data (G_OBJECT (menu_item),
-				   "rb-query-creator-value", GINT_TO_POINTER (options[i].val));
+				   "rb-query-creator-value", GINT_TO_POINTER (i));
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+		gtk_widget_show (menu_item);
+	}
+
+	if (activate_first)
+		gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 0);
+
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
+}
+
+static void
+setup_criteria_option_menu (GtkWidget *option_menu,
+			    const RBQueryCreatorCriteriaOption *options,
+			    int length, gboolean activate_first)
+{
+	GtkWidget *menu;
+	GtkWidget *menu_item;
+	int i;
+
+	menu = gtk_menu_new ();
+	gtk_widget_show (menu);
+    
+	for (i = 0; i < length; i++) {
+		menu_item = gtk_menu_item_new_with_label (_(options[i].name));
+		g_object_set_data (G_OBJECT (menu_item),
+				   "rb-query-creator-value", GINT_TO_POINTER (i));
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
 		gtk_widget_show (menu_item);
 	}
@@ -520,8 +702,8 @@ setup_option_menu (GtkWidget *option_menu,
 }
 
 static GtkWidget*
-create_option_menu (const RBQueryCreatorOption *options,
-		    int length, gboolean activate_first)
+create_property_option_menu (const RBQueryCreatorPropertyOption *options,
+			     int length, gboolean activate_first)
 {
 	GtkWidget *menu;
 	GtkWidget *option_menu;
@@ -531,7 +713,24 @@ create_option_menu (const RBQueryCreatorOption *options,
     
 	option_menu = gtk_option_menu_new ();
 
-	setup_option_menu (option_menu, options, length, activate_first);
+	setup_property_option_menu (option_menu, options, length, activate_first);
+  
+	return option_menu;
+}
+
+static GtkWidget*
+create_criteria_option_menu (const RBQueryCreatorCriteriaOption *options,
+			     int length, gboolean activate_first)
+{
+	GtkWidget *menu;
+	GtkWidget *option_menu;
+  
+	menu = gtk_menu_new ();
+	gtk_widget_show (menu);
+    
+	option_menu = gtk_option_menu_new ();
+
+	setup_criteria_option_menu (option_menu, options, length, activate_first);
   
 	return option_menu;
 }
