@@ -2,6 +2,7 @@
  *  arch-tag: Implementation of local song properties dialog
  *
  *  Copyright (C) 2002 Olivier Martin <omartin@ifrance.com>
+ *  Copyright (C) 2003 Colin Walters <walters@verbum.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,23 +23,10 @@
 #include <config.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnome/gnome-i18n.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkmenu.h>
-#include <gtk/gtkoptionmenu.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtkbbox.h>
-#include <gtk/gtkcombo.h>
-#include <gtk/gtktextview.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktable.h>
+#include <gtk/gtk.h>
 #include <glade/glade.h>
 #include <string.h>
 #include <time.h>
-#include <monkey-media-stream-info.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
 #include "rhythmdb.h"
@@ -64,19 +52,11 @@ static void rb_song_info_response_cb (GtkDialog *dialog,
 				      int response_id,
 				      RBSongInfo *song_info);
 static void rb_song_info_populate_dialog (RBSongInfo *song_info);
-static void rb_song_info_update_title (RBSongInfo *song_info);
-static void rb_song_info_update_track (RBSongInfo *song_info);
-static void rb_song_info_update_quality (RBSongInfo *song_info);
 static void rb_song_info_update_duration (RBSongInfo *song_info);
 static void rb_song_info_update_location (RBSongInfo *song_info);
-static void rb_song_info_update_genre (RBSongInfo *song_info);
 static void rb_song_info_update_play_count (RBSongInfo *song_info);
 static void rb_song_info_update_last_played (RBSongInfo *song_info);
-static void rb_song_info_update_encoding (RBSongInfo *song_info);
-static void rb_song_info_update_entry (RBSongInfo *song_info,
-		                       MonkeyMediaStreamInfoField field,
-		                       GtkWidget *widget);
-static void rb_song_info_update_comments (RBSongInfo *song_info);
+static void rb_song_info_update_bitrate (RBSongInfo *song_info);
 static void rb_song_info_update_buttons (RBSongInfo *song_info);
 static void rb_song_info_update_rating (RBSongInfo *song_info);
 static gboolean rb_song_info_update_current_values (RBSongInfo *song_info);
@@ -88,10 +68,10 @@ static void rb_song_info_forward_clicked_cb (GtkWidget *button,
 static void rb_song_info_view_changed_cb (RBEntryView *entry_view,
 					  RBSongInfo *song_info);
 static void rb_song_info_rated_cb (RBRating *rating,
-				   int score,
+				   double score,
 				   RBSongInfo *song_info);
 static void rb_song_info_mnemonic_cb (GtkWidget *target);
-static void cleanup (RBSongInfo *dlg);
+static void rb_song_info_sync_entries (RBSongInfo *dialog);
 
 struct RBSongInfoPrivate
 {
@@ -100,31 +80,43 @@ struct RBSongInfoPrivate
 
 	/* information on the displayed song */
 	RhythmDBEntry *current_entry;
-	MonkeyMediaStreamInfo *current_info;
+	GList *selected_entries;
+
+	gboolean editable;
 
 	/* the dialog widgets */
 	GtkWidget   *backward;
 	GtkWidget   *forward;
 
-	GtkWidget   *title;
-	GtkWidget   *artist;
-	GtkWidget   *album;
-	GtkWidget   *date;
-	GtkWidget   *track_cur;
-	GtkWidget   *track_max;
-	GtkWidget   *genre;
-	GtkWidget   *comments;
+	GtkWidget   *notebook;
 
-	GtkWidget   *quality;
+	GtkWidget   *title;
+	GtkWidget   *title_label;
+	GtkWidget   *artist;
+	GtkWidget   *artist_label;
+	GtkWidget   *album;
+	GtkWidget   *album_label;
+	GtkWidget   *genre;
+	GtkWidget   *genre_label;
+	GtkWidget   *track_cur;
+	GtkWidget   *track_cur_label;
+
+	GtkWidget   *bitrate;
 	GtkWidget   *duration;
 	GtkWidget   *name;
 	GtkWidget   *location;
 	GtkWidget   *play_count;
 	GtkWidget   *last_played;
-	GtkWidget   *encoding;
 	GtkWidget   *rating;
 };
 
+enum
+{
+	PRE_METADATA_CHANGE,
+	POST_METADATA_CHANGE,
+	LAST_SIGNAL
+};
+
 enum 
 {
 	PROP_0,
@@ -133,6 +125,8 @@ enum
 
 static GObjectClass *parent_class = NULL;
 
+static guint rb_song_info_signals[LAST_SIGNAL] = { 0 };
+
 GType
 rb_song_info_get_type (void)
 {
@@ -180,13 +174,35 @@ rb_song_info_class_init (RBSongInfoClass *klass)
 					                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
 	object_class->finalize = rb_song_info_finalize;
+
+	rb_song_info_signals[PRE_METADATA_CHANGE] =
+		g_signal_new ("pre-metadata-change",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBSongInfoClass, pre_metadata_change),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_POINTER);
+
+	rb_song_info_signals[POST_METADATA_CHANGE] =
+		g_signal_new ("post-metadata-change",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBSongInfoClass, post_metadata_change),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__POINTER,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_POINTER);
 }
 
 static void
 rb_song_info_init (RBSongInfo *song_info)
 {
 	GladeXML *xml;
-	GtkWidget *close, *cont;
+	GtkWidget *close;
 	
 	/* create the dialog and some buttons backward - forward - close */
 	song_info->priv = g_new0 (RBSongInfoPrivate, 1);
@@ -233,60 +249,39 @@ rb_song_info_init (RBSongInfo *song_info)
 				song_info);
 	glade_xml_signal_autoconnect (xml);
 
+	song_info->priv->notebook = glade_xml_get_widget (xml, "song_info_vbox");
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (song_info)->vbox),
-			   glade_xml_get_widget (xml, "song_info_vbox"));
+			   song_info->priv->notebook);
 
 	/* get the widgets from the XML */
 	song_info->priv->title         = glade_xml_get_widget (xml, "song_info_title");
+	song_info->priv->title_label   = glade_xml_get_widget (xml, "title_label");
 	song_info->priv->artist        = glade_xml_get_widget (xml, "song_info_artist");
 	song_info->priv->album         = glade_xml_get_widget (xml, "song_info_album");
-	song_info->priv->date          = glade_xml_get_widget (xml, "song_info_date");
 	song_info->priv->track_cur     = glade_xml_get_widget (xml, "song_info_track_cur");
-	song_info->priv->track_max     = glade_xml_get_widget (xml, "song_info_track_max");
+	song_info->priv->track_cur_label = glade_xml_get_widget (xml, "trackn_label");
 	song_info->priv->genre         = glade_xml_get_widget (xml, "song_info_genre");
-	song_info->priv->comments      = glade_xml_get_widget (xml, "song_info_comments");
-	song_info->priv->quality       = glade_xml_get_widget (xml, "song_info_quality");
+	song_info->priv->bitrate       = glade_xml_get_widget (xml, "song_info_bitrate");
 	song_info->priv->duration      = glade_xml_get_widget (xml, "song_info_duration");
-	cont = glade_xml_get_widget (xml, "song_info_location_container");
-	song_info->priv->location = rb_ellipsizing_label_new ("");
-	gtk_misc_set_alignment (GTK_MISC (song_info->priv->location), 0.0, 0.5);
-	gtk_label_set_selectable (GTK_LABEL (song_info->priv->location), TRUE);
-	gtk_container_add (GTK_CONTAINER (cont), song_info->priv->location);
-	gtk_widget_show (song_info->priv->location);
+	song_info->priv->location = glade_xml_get_widget (xml, "song_info_location");
 	song_info->priv->play_count    = glade_xml_get_widget (xml, "song_info_playcount");
 	song_info->priv->last_played   = glade_xml_get_widget (xml, "song_info_lastplayed");
-	song_info->priv->encoding      = glade_xml_get_widget (xml, "song_info_encoding");
-	cont = glade_xml_get_widget (xml, "song_info_name_container");
-	song_info->priv->name = rb_ellipsizing_label_new ("");
-	gtk_misc_set_alignment (GTK_MISC (song_info->priv->name), 0.0, 0.5);
-	gtk_label_set_selectable (GTK_LABEL (song_info->priv->name), TRUE);
-	gtk_container_add (GTK_CONTAINER (cont), song_info->priv->name);
-	gtk_widget_show (song_info->priv->name);
+	song_info->priv->name = glade_xml_get_widget (xml, "song_info_name");
 
 	/* We add now the Pango attributes (look at bug #99867 and #97061) */
 	{
 		gchar *str_final;
 		GtkWidget *label;
 
-		label = glade_xml_get_widget (xml, "comments_label");
-		str_final = g_strdup_printf ("<b>%s</b>",
-					     gtk_label_get_label GTK_LABEL (label));
-		gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str_final);
-		g_free (str_final);
-
-		label = glade_xml_get_widget (xml, "date_label");
-		str_final = g_strdup_printf ("<b>%s</b>",
-					     gtk_label_get_label GTK_LABEL (label));
-		gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str_final);
-		g_free (str_final);
-
 		label = glade_xml_get_widget (xml, "album_label");
+		song_info->priv->album_label = label;
 		str_final = g_strdup_printf ("<b>%s</b>",
 					     gtk_label_get_label GTK_LABEL (label));
 		gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str_final);
 		g_free (str_final);
 
 		label = glade_xml_get_widget (xml, "artist_label");
+		song_info->priv->artist_label = label;
 		str_final = g_strdup_printf ("<b>%s</b>",
 					     gtk_label_get_label GTK_LABEL (label));
 		gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str_final);
@@ -299,6 +294,7 @@ rb_song_info_init (RBSongInfo *song_info)
 		g_free (str_final);
 
 		label = glade_xml_get_widget (xml, "genre_label");
+		song_info->priv->genre_label = label;
 		str_final = g_strdup_printf ("<b>%s</b>",
 					     gtk_label_get_label GTK_LABEL (label));
 		gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str_final);
@@ -346,29 +342,14 @@ rb_song_info_init (RBSongInfo *song_info)
 		gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str_final);
 		g_free (str_final);
 
-		label = glade_xml_get_widget (xml, "quality_label");
+		label = glade_xml_get_widget (xml, "bitrate_label");
 		str_final = g_strdup_printf ("<b>%s</b>",
 					     gtk_label_get_label GTK_LABEL (label));
 		gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str_final);
 		g_free (str_final);
 
-		label = glade_xml_get_widget (xml, "encoding_label");
-		str_final = g_strdup_printf ("<b>%s</b>",
-					     gtk_label_get_label GTK_LABEL (label));
-		gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), str_final);
-		g_free (str_final);
-		
 	}
 
-	/* make those fields not editable for now */
-	gtk_entry_set_editable (GTK_ENTRY (song_info->priv->title), FALSE);
-	gtk_entry_set_editable (GTK_ENTRY (song_info->priv->artist), FALSE);
-	gtk_entry_set_editable (GTK_ENTRY (song_info->priv->album), FALSE);
-	gtk_entry_set_editable (GTK_ENTRY (song_info->priv->date), FALSE);
-	gtk_entry_set_editable (GTK_ENTRY (song_info->priv->track_cur), FALSE);
-	gtk_entry_set_editable (GTK_ENTRY (song_info->priv->track_max), FALSE);
-	gtk_text_view_set_editable (GTK_TEXT_VIEW (song_info->priv->comments), FALSE);
-
 	/* whenever you press a mnemonic, the associated GtkEntry's text gets highlighted */
 	g_signal_connect (G_OBJECT (song_info->priv->title),
 			  "mnemonic-activate",
@@ -382,7 +363,7 @@ rb_song_info_init (RBSongInfo *song_info)
 			  "mnemonic-activate",
 			  G_CALLBACK (rb_song_info_mnemonic_cb),
 			  NULL);
-	g_signal_connect (G_OBJECT (song_info->priv->date),
+	g_signal_connect (G_OBJECT (song_info->priv->genre),
 			  "mnemonic-activate",
 			  G_CALLBACK (rb_song_info_mnemonic_cb),
 			  NULL);
@@ -390,18 +371,6 @@ rb_song_info_init (RBSongInfo *song_info)
 			  "mnemonic-activate",
 			  G_CALLBACK (rb_song_info_mnemonic_cb),
 			  NULL);
-	g_signal_connect (G_OBJECT (song_info->priv->track_max),
-			  "mnemonic-activate",
-			  G_CALLBACK (rb_song_info_mnemonic_cb),
-			  NULL);
-	g_signal_connect (G_OBJECT (GTK_COMBO (song_info->priv->genre)->entry),
-			  "mnemonic-activate",
-			  G_CALLBACK (rb_song_info_mnemonic_cb),
-			  NULL);
-	g_signal_connect (G_OBJECT (song_info->priv->comments),
-			  "mnemonic-activate",
-			  G_CALLBACK (rb_song_info_mnemonic_cb),
-			  NULL);
 
 	/* this widget has to be customly created */
 	song_info->priv->rating = GTK_WIDGET (rb_rating_new ());
@@ -430,11 +399,6 @@ rb_song_info_finalize (GObject *object)
 
 	g_return_if_fail (song_info->priv != NULL);
 
-	if (song_info->priv->current_info != NULL)
-	{
-		cleanup (song_info);
-	}
-
 	g_free (song_info->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -451,20 +415,20 @@ rb_song_info_set_property (GObject *object,
 	switch (prop_id)
 	{
 	case PROP_ENTRY_VIEW:
-		{
-			RBEntryView *entry_view = g_value_get_object (value);
-			song_info->priv->entry_view = entry_view;
-			g_object_get (G_OBJECT (entry_view), "db", &song_info->priv->db, NULL);
+	{
+		RBEntryView *entry_view = g_value_get_object (value);
+		song_info->priv->entry_view = entry_view;
+		g_object_get (G_OBJECT (entry_view), "db", &song_info->priv->db, NULL);
+		
+		rb_song_info_update_current_values (song_info);
 
-			rb_song_info_update_current_values (song_info);
-
-			g_signal_connect_object (G_OBJECT (entry_view),
-						 "changed",
-						 G_CALLBACK (rb_song_info_view_changed_cb),
-						 song_info,
+		g_signal_connect_object (G_OBJECT (entry_view),
+					 "changed",
+					 G_CALLBACK (rb_song_info_view_changed_cb),
+					 song_info,
 						 G_CONNECT_AFTER);
-		}
-		break;
+	}
+	break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -514,13 +478,16 @@ rb_song_info_response_cb (GtkDialog *dialog,
 			  int response_id,
 			  RBSongInfo *song_info)
 {
-	if (response_id == GTK_RESPONSE_CLOSE)
+	if (response_id == GTK_RESPONSE_CLOSE) {
+		if (song_info->priv->editable)
+			rb_song_info_sync_entries (RB_SONG_INFO (dialog));
 		gtk_widget_destroy (GTK_WIDGET (dialog));
+	}
 }
 
 static void
 rb_song_info_rated_cb (RBRating *rating,
-		       int score,
+		       double score,
 		       RBSongInfo *song_info)
 {
 	GValue value = { 0, };
@@ -530,8 +497,8 @@ rb_song_info_rated_cb (RBRating *rating,
 	g_return_if_fail (score >= 0 && score <= 5 );
 
 	/* set the new value for the song */
-	g_value_init (&value, G_TYPE_INT);
-	g_value_set_int (&value, score);
+	g_value_init (&value, G_TYPE_DOUBLE);
+	g_value_set_double (&value, score);
 	rhythmdb_write_lock (song_info->priv->db);
 	rhythmdb_entry_set (song_info->priv->db,
 			    song_info->priv->current_entry,
@@ -562,319 +529,96 @@ rb_song_info_mnemonic_cb (GtkWidget *target)
 static void 
 rb_song_info_populate_dialog (RBSongInfo *song_info)
 {
+	gint num;
+	const char *text = NULL;
+	char *tmp;
 	/* update the buttons sensitivity */
 	rb_song_info_update_buttons (song_info);
+
+	if (!song_info->priv->current_entry)
+		return;
 	
-	/* update the fields values */
-	rb_song_info_update_title (song_info);
-	rb_song_info_update_entry (song_info,
-				   MONKEY_MEDIA_STREAM_INFO_FIELD_ARTIST,
-				   song_info->priv->artist);
-	rb_song_info_update_entry (song_info,
-				   MONKEY_MEDIA_STREAM_INFO_FIELD_ALBUM,
-				   song_info->priv->album);
-	rb_song_info_update_entry (song_info,
-				   MONKEY_MEDIA_STREAM_INFO_FIELD_DATE,
-				   song_info->priv->date);
-	rb_song_info_update_track (song_info);
-	rb_song_info_update_genre (song_info);
-	rb_song_info_update_comments (song_info);
-	rb_song_info_update_quality (song_info);
+	rhythmdb_read_lock (song_info->priv->db);
+
+	text = rhythmdb_entry_get_string (song_info->priv->db,
+					  song_info->priv->current_entry,
+					  RHYTHMDB_PROP_TITLE);
+	gtk_entry_set_text (GTK_ENTRY (song_info->priv->title), text);
+	
+	tmp = g_strdup_printf (_("%s Properties"), text);
+	gtk_window_set_title (GTK_WINDOW (song_info), tmp);
+	g_free (tmp);
+
+	text = rhythmdb_entry_get_string (song_info->priv->db,
+					  song_info->priv->current_entry,
+					  RHYTHMDB_PROP_ARTIST);
+	gtk_entry_set_text (GTK_ENTRY (song_info->priv->artist), text);
+	text = rhythmdb_entry_get_string (song_info->priv->db,
+					  song_info->priv->current_entry,
+					  RHYTHMDB_PROP_ALBUM);
+	gtk_entry_set_text (GTK_ENTRY (song_info->priv->album), text);
+	text = rhythmdb_entry_get_string (song_info->priv->db,
+					  song_info->priv->current_entry,
+					  RHYTHMDB_PROP_GENRE);
+	gtk_entry_set_text (GTK_ENTRY (song_info->priv->genre), text);
+
+	num = rhythmdb_entry_get_int (song_info->priv->db,
+				      song_info->priv->current_entry,
+				      RHYTHMDB_PROP_TRACK_NUMBER);
+	if (num > 0)
+		tmp = g_strdup_printf ("%.2d", num);
+	else
+		tmp = g_strdup (_("Never"));
+	gtk_entry_set_text (GTK_ENTRY (song_info->priv->track_cur),
+			    tmp);
+	g_free (tmp);
+	num = rhythmdb_entry_get_int (song_info->priv->db,
+				      song_info->priv->current_entry,
+				      RHYTHMDB_PROP_BITRATE);
+	if (num > 0)
+		tmp = g_strdup_printf ("%d", num);
+	else
+		tmp = g_strdup (_("Unknown"));
+	gtk_label_set_text (GTK_LABEL (song_info->priv->bitrate),
+			    tmp);
+	g_free (tmp);
+
 	rb_song_info_update_duration (song_info);
 	rb_song_info_update_location (song_info);
 	rb_song_info_update_play_count (song_info);
 	rb_song_info_update_last_played (song_info);
-	rb_song_info_update_encoding (song_info);
+	rb_song_info_update_bitrate (song_info);
 	rb_song_info_update_rating (song_info);
+	rhythmdb_read_unlock (song_info->priv->db);
 }
 
 static void
-rb_song_info_update_title (RBSongInfo *song_info)
-{
-	GValue value = { 0, };
-	const char *text = NULL;
-	gboolean res = FALSE;
-
-	if (song_info->priv->current_info != NULL)
-	{
-		res = monkey_media_stream_info_get_value (song_info->priv->current_info, 
-							  MONKEY_MEDIA_STREAM_INFO_FIELD_TITLE,
-							  0, &value);
-	}
-
-	if (res == TRUE)
-	{
-		text = g_value_get_string (&value);
-	}
-	else
-	{
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_set_string (&value, "");
-	}
-
-	if (text != NULL)
-	{
-		char *tmp;
-		
-		gtk_entry_set_text (GTK_ENTRY (song_info->priv->title), text);
-
-		tmp = g_strdup_printf (_("%s Properties"), text);
-		gtk_window_set_title (GTK_WINDOW (song_info), tmp);
-		g_free (tmp);
-	}
-	else
-	{
-		if (song_info->priv->current_entry != NULL)
-		{
-			const char *url;
-			char *tmp;
-
-			rhythmdb_read_lock (song_info->priv->db);
-
-			url = rhythmdb_entry_get_string (song_info->priv->db,
-							 song_info->priv->current_entry,
-							 RHYTHMDB_PROP_LOCATION);
-			rhythmdb_read_unlock (song_info->priv->db);
-
-			tmp = g_strdup_printf (_("%s Properties"), url);
-			gtk_window_set_title (GTK_WINDOW (song_info), tmp);
-			g_free (tmp);
-		}
-		else
-			gtk_window_set_title (GTK_WINDOW (song_info), _("Song Properties"));
-	}
-
-	g_value_unset (&value);
-}
-
-static void
-rb_song_info_update_comments (RBSongInfo *song_info)
-{
-	GValue value = { 0, };
-	const char *text = NULL;
-	GtkTextBuffer *buffer;
-	GtkTextIter start, end;
-	gboolean res = FALSE;
-
-	if (song_info->priv->current_info != NULL)
-	{
-		res = monkey_media_stream_info_get_value (song_info->priv->current_info, 
-							  MONKEY_MEDIA_STREAM_INFO_FIELD_COMMENT,
-							  0, &value);
-	}
-	if (res == TRUE)
-	{
-		text = g_value_get_string (&value); 
-	}
-	else
-	{
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_set_string (&value, "");
-	}
-
-	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (song_info->priv->comments));
-
-	gtk_text_buffer_get_bounds (buffer, &start, &end);
-
-	gtk_text_buffer_delete (buffer, &start, &end);
-
-	if (text != NULL)
-		gtk_text_buffer_insert (buffer, &start, text, -1);
-
-	g_value_unset (&value);
-}
-
-static void
-rb_song_info_update_track (RBSongInfo *song_info)
-{
-	int val = 0;
-	GValue value = { 0, };
-	gboolean res = FALSE;
-
-	if (song_info->priv->current_info != NULL) {
-		res = monkey_media_stream_info_get_value (song_info->priv->current_info, 
-							  MONKEY_MEDIA_STREAM_INFO_FIELD_TRACK_NUMBER,
-							  0, &value);
-	}
-
-	if (res == TRUE) {
-		val = g_value_get_int (&value);
-	} else {
-		g_value_init (&value, G_TYPE_INT);
-		g_value_set_int (&value, 0);
-	}
-
-	if (val >= 0) {
-		char *tmp;
-		tmp = g_strdup_printf ("%.2d", val);
-		gtk_entry_set_text (GTK_ENTRY (song_info->priv->track_cur), tmp);
-		g_free (tmp);
-	}
-
-	g_value_unset (&value);
-
-	res = FALSE;
-	if (song_info->priv->current_info != NULL) {
-		res = monkey_media_stream_info_get_value (song_info->priv->current_info, 
-							  MONKEY_MEDIA_STREAM_INFO_FIELD_MAX_TRACK_NUMBER,
-							  0, &value);
-	}
-
-	if (res == TRUE) {
-		val = g_value_get_int (&value);
-	} else {
-		val = 0;
-		g_value_init (&value, G_TYPE_INT);
-		g_value_set_int (&value, 0);
-	}
-
-	if (val > 0) {
-		char *tmp;
-		tmp = g_strdup_printf ("%.2d", val);
-		gtk_entry_set_text (GTK_ENTRY (song_info->priv->track_max), tmp);
-		g_free (tmp);
-	} else {
-		gtk_entry_set_text (GTK_ENTRY (song_info->priv->track_max), "");
-	}
-
-	g_value_unset (&value);
-}
-
-static void
-rb_song_info_update_entry (RBSongInfo *song_info,
-		           MonkeyMediaStreamInfoField field,
-		           GtkWidget *widget)
-{
-	GValue value = { 0, };
-	const char *text = NULL;
-	gboolean res = FALSE;
-
-	if (song_info->priv->current_info != NULL)
-	{
-		res = monkey_media_stream_info_get_value (song_info->priv->current_info, 
-							  field, 0, &value);
-	}
-
-	if (res == TRUE)
-	{
-		text = g_value_get_string (&value);
-	}
-	else
-	{
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_set_string (&value, "");
-	}
-
-	if (text != NULL)
-		gtk_entry_set_text (GTK_ENTRY (widget), text);
-
-	g_value_unset (&value);
-}
-
-static void
-rb_song_info_update_genre (RBSongInfo *song_info)
-{
-	const char *genre = NULL;
-	GValue value = { 0, };
-	gboolean res = FALSE;
-
-	if (song_info->priv->current_info != NULL)
-	{
-		res = monkey_media_stream_info_get_value (song_info->priv->current_info, 
-							  MONKEY_MEDIA_STREAM_INFO_FIELD_GENRE,
-							  0, &value);
-	}
-
-	if (res == TRUE)
-	{
-		genre = g_value_get_string (&value);
-	}
-	else
-	{
-		g_value_init (&value, G_TYPE_STRING);
-		g_value_set_string (&value, "");
-	}
-
-	gtk_combo_set_popdown_strings (GTK_COMBO (song_info->priv->genre),
-				       monkey_media_stream_info_list_all_genres ());
-
-	if (genre != NULL)
-		gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (song_info->priv->genre)->entry),
-				    genre);
-	
-	g_value_unset (&value);
-}
-
-static void
-rb_song_info_update_quality (RBSongInfo *song_info)
+rb_song_info_update_bitrate (RBSongInfo *song_info)
 {
 	char *text = NULL;
-	GValue value = { 0, };
-	gboolean res = FALSE;
-
-	if (song_info->priv->current_info != NULL)
-	{
-		res = monkey_media_stream_info_get_value (song_info->priv->current_info, 
-							  MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_QUALITY,
-							  0, &value);
-	}
-
-	if (res == TRUE)
-	{
-		text = monkey_media_audio_quality_to_string (g_value_get_enum (&value));
-	}
-	else
-	{
-		g_value_init (&value, G_TYPE_INT);
-		g_value_set_string (&value, 0);
-	}
-
-	if (text != NULL)
-	{
-		gtk_label_set_text (GTK_LABEL (song_info->priv->quality), _(text));
-		g_free (text);
-	}
-
-	g_value_unset (&value);
+	int bitrate = 0;
+	bitrate = rhythmdb_entry_get_int (song_info->priv->db,
+					     song_info->priv->current_entry,
+					     RHYTHMDB_PROP_BITRATE);
+	text = g_strdup_printf ("%d", bitrate);
+	gtk_label_set_text (GTK_LABEL (song_info->priv->duration), text);
+	g_free (text);
 }
 
 static void
 rb_song_info_update_duration (RBSongInfo *song_info)
 {
-	GValue value = { 0, };
 	char *text = NULL;
 	long duration = 0;
 	int minutes, seconds;
-	gboolean res = FALSE;
-
-	if (song_info->priv->current_info != NULL)
-	{
-		res = monkey_media_stream_info_get_value (song_info->priv->current_info, 
-							  MONKEY_MEDIA_STREAM_INFO_FIELD_DURATION,
-							  0, &value);
-	}
-
-	if (res == TRUE)
-	{
-		duration = g_value_get_long (&value);
-		minutes = duration / 60;
-		seconds = duration % 60;
-		text = g_strdup_printf ("%d:%02d", minutes, seconds);
-	}
-	else
-	{
-		g_value_init (&value, G_TYPE_LONG);
-		g_value_set_long (&value, 0);
-	}
-
-
-	if (text != NULL)
-	{
-		gtk_label_set_text (GTK_LABEL (song_info->priv->duration), text);
-		g_free (text);
-	}
-
-	g_value_unset (&value);
+	duration = rhythmdb_entry_get_long (song_info->priv->db,
+					    song_info->priv->current_entry,
+					    RHYTHMDB_PROP_DURATION);
+	minutes = duration / 60;
+	seconds = duration % 60;
+	text = g_strdup_printf ("%d:%02d", minutes, seconds);
+	gtk_label_set_text (GTK_LABEL (song_info->priv->duration), text);
+	g_free (text);
 }
 
 static void
@@ -891,18 +635,15 @@ rb_song_info_update_location (RBSongInfo *song_info)
 					  RHYTHMDB_PROP_LOCATION);
 	rhythmdb_read_unlock (song_info->priv->db);
 
-	if (text != NULL)
-	{
+	if (text != NULL) {
 		char *tmp;
 		
 		basename = g_path_get_basename (text);
 		tmp = gnome_vfs_unescape_string_for_display (basename);
 		g_free (basename);
 
-		if (tmp != NULL)
-		{
-			rb_ellipsizing_label_set_mode (RB_ELLIPSIZING_LABEL (song_info->priv->name), RB_ELLIPSIZE_END);
-			rb_ellipsizing_label_set_text (RB_ELLIPSIZING_LABEL (song_info->priv->name), tmp);
+		if (tmp != NULL) {
+			gtk_entry_set_text (GTK_ENTRY (song_info->priv->name), tmp);
 		}
 
 		g_free (tmp);
@@ -923,8 +664,7 @@ rb_song_info_update_location (RBSongInfo *song_info)
 		}
 		g_free (desktopdir);
 		
-		rb_ellipsizing_label_set_mode (RB_ELLIPSIZING_LABEL (song_info->priv->location), RB_ELLIPSIZE_END);
-		rb_ellipsizing_label_set_text (RB_ELLIPSIZING_LABEL (song_info->priv->location), tmp);
+		gtk_entry_set_text (GTK_ENTRY (song_info->priv->location), tmp);
 		g_free (tmp);
 	}
 }
@@ -941,10 +681,7 @@ rb_song_info_backward_clicked_cb (GtkWidget *button,
 	rb_entry_view_scroll_to_entry (song_info->priv->entry_view,
 				       song_info->priv->current_entry);
 
-	cleanup (song_info);
-
-	if (rb_song_info_update_current_values (song_info) == TRUE)
-		rb_song_info_populate_dialog (song_info);
+	rb_song_info_populate_dialog (song_info);
 }
 
 static void
@@ -959,10 +696,7 @@ rb_song_info_forward_clicked_cb (GtkWidget *button,
 	rb_entry_view_scroll_to_entry (song_info->priv->entry_view,
 				       song_info->priv->current_entry);
 
-	cleanup (song_info);
-
-	if (rb_song_info_update_current_values (song_info) == TRUE)
-		rb_song_info_populate_dialog (song_info);
+	rb_song_info_populate_dialog (song_info);
 }
 
 /*
@@ -975,16 +709,17 @@ rb_song_info_update_buttons (RBSongInfo *song_info)
 
 	g_return_if_fail (song_info != NULL);
 	g_return_if_fail (song_info->priv->entry_view != NULL);
-	g_return_if_fail (song_info->priv->current_entry != NULL);
 
 	/* backward */
-	entry = rb_entry_view_get_previous_from_entry (song_info->priv->entry_view,
-						      song_info->priv->current_entry);
+	if (song_info->priv->current_entry)
+		entry = rb_entry_view_get_previous_from_entry (song_info->priv->entry_view,
+							       song_info->priv->current_entry);
 	
 	gtk_widget_set_sensitive (song_info->priv->backward, entry != NULL);
 	/* forward */
-	entry = rb_entry_view_get_next_from_entry (song_info->priv->entry_view,
-						   song_info->priv->current_entry);
+	if (song_info->priv->current_entry)
+		entry = rb_entry_view_get_next_from_entry (song_info->priv->entry_view,
+							   song_info->priv->current_entry);
 
 	gtk_widget_set_sensitive (song_info->priv->forward, entry != NULL);
 }
@@ -997,42 +732,59 @@ rb_song_info_view_changed_cb (RBEntryView *entry_view,
 	rb_song_info_update_buttons (song_info);
 }
 
-/*
- * rb_song_info_update_current_values: get the selection from
- * the node view and then updates current_node & current_info
- */
 static gboolean
 rb_song_info_update_current_values (RBSongInfo *song_info)
 {
-	const char *url;
-	RhythmDBEntry *entry = NULL;
-	MonkeyMediaStreamInfo *info;
 	GList *selected_entries;
+	GList *tem;
+	gboolean editable = TRUE;
+	gboolean multiselect = FALSE;
+
+	g_list_free (song_info->priv->selected_entries);
 
 	selected_entries = rb_entry_view_get_selected_entries (song_info->priv->entry_view);
 
-	if ((selected_entries == NULL) ||
-	    (selected_entries->data == NULL))
-	{
-		song_info->priv->current_info = NULL;
+	if ((selected_entries == NULL) || (selected_entries->data == NULL)) {
 		song_info->priv->current_entry = NULL;
+		song_info->priv->selected_entries = NULL;
 		gtk_widget_destroy (GTK_WIDGET (song_info));
-
 		return FALSE;
 	}
 
-	song_info->priv->current_entry = entry = selected_entries->data;
-
-	/* get the stream info */
 	rhythmdb_read_lock (song_info->priv->db);
-	url = rhythmdb_entry_get_string (song_info->priv->db,
-					  entry,
-					  RHYTHMDB_PROP_LOCATION);
+	for (tem = selected_entries; tem; tem = tem->next)
+		if (!rhythmdb_entry_is_editable (song_info->priv->db,
+						 selected_entries->data)) {
+			editable = FALSE;
+			multiselect = TRUE; /* Just to avoid more conditionals */
+			break;
+		}
 	rhythmdb_read_unlock (song_info->priv->db);
 
-	info = monkey_media_stream_info_new (url, NULL);
-	song_info->priv->current_info = info;
+	if (selected_entries->next == NULL) {
+		song_info->priv->current_entry = selected_entries->data;
+		song_info->priv->selected_entries = NULL;
+		g_list_free (selected_entries);
+	} else {
+		song_info->priv->current_entry = NULL;
+		song_info->priv->selected_entries = selected_entries;
+		multiselect = TRUE;
+	}
 
+	if (multiselect && editable) 
+		gtk_notebook_remove_page (GTK_NOTEBOOK (song_info->priv->notebook), 1);
+	gtk_editable_set_editable  (GTK_EDITABLE (song_info->priv->title), editable);
+	gtk_widget_set_sensitive (song_info->priv->title_label, !multiselect);
+	gtk_editable_set_editable  (GTK_EDITABLE (song_info->priv->artist), editable);
+	gtk_widget_set_sensitive (song_info->priv->artist_label, editable);
+	gtk_editable_set_editable  (GTK_EDITABLE (song_info->priv->album), editable);
+	gtk_widget_set_sensitive (song_info->priv->album_label, editable);
+	gtk_editable_set_editable  (GTK_EDITABLE (song_info->priv->genre), editable);
+	gtk_widget_set_sensitive (song_info->priv->genre_label, editable);
+	gtk_editable_set_editable  (GTK_EDITABLE (song_info->priv->track_cur), !multiselect);
+	gtk_widget_set_sensitive (song_info->priv->track_cur_label, !multiselect);
+
+	song_info->priv->editable = editable;
 	return TRUE;
 }
 
@@ -1040,139 +792,144 @@ static void
 rb_song_info_update_play_count (RBSongInfo *song_info)
 {
 	char *text;
-
-	rhythmdb_read_lock (song_info->priv->db);
-
 	text = g_strdup_printf ("%d", rhythmdb_entry_get_int (song_info->priv->db,
 							      song_info->priv->current_entry,
 							      RHYTHMDB_PROP_PLAY_COUNT));
-	rhythmdb_read_unlock (song_info->priv->db);
-
 	gtk_label_set_text (GTK_LABEL (song_info->priv->play_count), text);
+	g_free (text);
 }
 
 static void
 rb_song_info_update_last_played (RBSongInfo *song_info)
 {
 	const char *str;
-	rhythmdb_read_lock (song_info->priv->db);
 	str = rhythmdb_entry_get_string (song_info->priv->db,
 					 song_info->priv->current_entry,
 					 RHYTHMDB_PROP_LAST_PLAYED_STR);
-	rhythmdb_read_unlock (song_info->priv->db);
+	if (!strcmp ("", str))
+		str = _("Never");
 	gtk_label_set_text (GTK_LABEL (song_info->priv->last_played), str);
 }
 
 static void
 rb_song_info_update_rating (RBSongInfo *song_info)
 {
-	guint rating;
+	gdouble rating;
 	
 	g_return_if_fail (RB_IS_SONG_INFO (song_info));
 
-	rhythmdb_read_lock (song_info->priv->db);
-	rating = rhythmdb_entry_get_int (song_info->priv->db,
-					 song_info->priv->current_entry,
-					 RHYTHMDB_PROP_RATING);
-	rhythmdb_read_unlock (song_info->priv->db);
+	rating = rhythmdb_entry_get_double (song_info->priv->db,
+					    song_info->priv->current_entry,
+					    RHYTHMDB_PROP_RATING);
 
 	g_object_set (G_OBJECT (song_info->priv->rating),
 		      "score", rating,
 		      NULL);
+
 }
 
 static void
-rb_song_info_update_encoding (RBSongInfo *song_info)
+rb_song_info_sync_entries_multiple (RBSongInfo *dialog)
 {
-	const char *text_codec = NULL;
-	int channels = -1, bitrate = -1;
-	GValue value = { 0, };
-	gboolean res = FALSE;
-	GString *string;
+	const char *genre = gtk_entry_get_text (GTK_ENTRY (dialog->priv->genre));
+	const char *artist = gtk_entry_get_text (GTK_ENTRY (dialog->priv->artist));
+	const char *album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));	
+	GValue val = {0,};
+	GList *tem;
 
-	if (song_info->priv->current_info != NULL)
-	{
-		res = monkey_media_stream_info_get_value (song_info->priv->current_info, 
-							  MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_AVERAGE_BIT_RATE,
-							  0, &value);
+	rhythmdb_write_lock (dialog->priv->db);
+
+	if (strlen (album) > 0) {
+		g_value_init (&val, G_TYPE_STRING);
+		g_value_set_string (&val, album);
+		for (tem = dialog->priv->selected_entries; tem; tem = tem->next)
+			rhythmdb_entry_set (dialog->priv->db,
+					    tem->data, RHYTHMDB_PROP_ALBUM, &val);
+		g_value_unset (&val);
+	}
+	
+	if (strlen (artist) > 0) {
+		g_value_init (&val, G_TYPE_STRING);
+		g_value_set_string (&val, artist);
+		for (tem = dialog->priv->selected_entries; tem; tem = tem->next)
+			rhythmdb_entry_set (dialog->priv->db,
+					    tem->data, RHYTHMDB_PROP_ARTIST, &val);
+		g_value_unset (&val);
 	}
 
-	if (res == TRUE)
-	{
-		bitrate = g_value_get_int (&value);
-		g_value_unset (&value);
+	if (strlen (genre) > 0) {
+		g_value_init (&val, G_TYPE_STRING);
+		g_value_set_string (&val, genre);
+		for (tem = dialog->priv->selected_entries; tem; tem = tem->next)
+			rhythmdb_entry_set (dialog->priv->db,
+					    tem->data, RHYTHMDB_PROP_GENRE, &val);
+		g_value_unset (&val);
 	}
-
-	res = FALSE;
-
-	if (song_info->priv->current_info != NULL)
-	{
-		res = monkey_media_stream_info_get_value (song_info->priv->current_info, 
-							  MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_CHANNELS,
-							  0, &value);
-	}
-
-	if (res == TRUE)
-	{
-		channels = g_value_get_int (&value);
-		g_value_unset (&value);
-	}
-
-	res = FALSE;
-
-	if (song_info->priv->current_info != NULL)
-	{
-		res = monkey_media_stream_info_get_value (song_info->priv->current_info, 
-							  MONKEY_MEDIA_STREAM_INFO_FIELD_AUDIO_CODEC_INFO,
-							  0, &value);
-	}
-
-	if (res == TRUE)
-	{
-		text_codec = g_value_get_string (&value);
-	}
-
-	string = g_string_new ("");
-
-	if (bitrate >= 0)
-	{
-		g_string_append_printf (string, _("%d kbps"), bitrate);
-	}
-
-	if (text_codec != NULL)
-	{
-		g_string_append (string, text_codec);
-	}
-
-	if (channels >= 0)
-	{
-		switch (channels)
-		{
-		case 1:
-			g_string_append (string, _(" (Mono)"));
-			break;
-		case 2:
-			g_string_append (string, _(" (Stereo)"));
-			break;
-		default:
-			g_string_append_printf (string,
-						ngettext (" (%d channel)", " (%d channel)", channels),
-						channels);
-			break;
-		}
-	}
-
-	gtk_label_set_text (GTK_LABEL (song_info->priv->encoding), string->str);
-		
-	g_string_free (string, TRUE);
-
-	if (res == TRUE)
-		g_value_unset (&value);
+	
+	rhythmdb_write_unlock (dialog->priv->db);
 }
 
 static void
-cleanup (RBSongInfo *dlg)
+rb_song_info_sync_entry_single (RBSongInfo *dialog)
 {
-	if (dlg->priv->current_info != NULL)
-		g_object_unref (G_OBJECT (dlg->priv->current_info));
+	const char *title = gtk_entry_get_text (GTK_ENTRY (dialog->priv->title));
+	const char *genre = gtk_entry_get_text (GTK_ENTRY (dialog->priv->genre));
+	const char *artist = gtk_entry_get_text (GTK_ENTRY (dialog->priv->artist));
+	const char *album = gtk_entry_get_text (GTK_ENTRY (dialog->priv->album));	
+	const char *tracknum_str = gtk_entry_get_text (GTK_ENTRY (dialog->priv->track_cur));
+	char *endptr;
+	gint tracknum;
+	GValue val = {0,};
+
+	tracknum = g_ascii_strtoull (tracknum_str, &endptr, 10);
+	if (endptr == tracknum_str)
+		tracknum = -1;
+
+	rhythmdb_write_lock (dialog->priv->db);
+	g_value_init (&val, G_TYPE_STRING);
+	g_value_set_string (&val, title);
+	rhythmdb_entry_set (dialog->priv->db,
+			    dialog->priv->current_entry, RHYTHMDB_PROP_TITLE, &val);
+	g_value_unset (&val);
+
+	g_value_init (&val, G_TYPE_INT);
+	g_value_set_int (&val, tracknum);
+	rhythmdb_entry_set (dialog->priv->db,
+			    dialog->priv->current_entry, RHYTHMDB_PROP_TRACK_NUMBER, &val);
+	g_value_unset (&val);
+
+	g_signal_emit (G_OBJECT (dialog), rb_song_info_signals[PRE_METADATA_CHANGE], 0,
+		       dialog->priv->current_entry);
+
+	g_value_init (&val, G_TYPE_STRING);
+	g_value_set_string (&val, album);
+	rhythmdb_entry_set (dialog->priv->db,
+			    dialog->priv->current_entry, RHYTHMDB_PROP_ALBUM, &val);
+	g_value_unset (&val);
+
+	g_value_init (&val, G_TYPE_STRING);
+	g_value_set_string (&val, artist);
+	rhythmdb_entry_set (dialog->priv->db,
+			    dialog->priv->current_entry, RHYTHMDB_PROP_ARTIST, &val);
+	g_value_unset (&val);
+
+	g_value_init (&val, G_TYPE_STRING);
+	g_value_set_string (&val, genre);
+	rhythmdb_entry_set (dialog->priv->db,
+			    dialog->priv->current_entry, RHYTHMDB_PROP_GENRE, &val);
+	g_value_unset (&val);
+
+	g_signal_emit (G_OBJECT (dialog), rb_song_info_signals[POST_METADATA_CHANGE], 0,
+		       dialog->priv->current_entry);
+
+	rhythmdb_write_unlock (dialog->priv->db);
+}	
+
+static void
+rb_song_info_sync_entries (RBSongInfo *dialog)
+{
+	if (dialog->priv->current_entry)
+		rb_song_info_sync_entry_single (dialog);
+	else
+		rb_song_info_sync_entries_multiple (dialog);
 }
